@@ -155,6 +155,17 @@ class LastUpdateStatusReasonCode(str):
     ImageDeleted = "ImageDeleted"
     ImageAccessDenied = "ImageAccessDenied"
     InvalidImage = "InvalidImage"
+    KMSKeyAccessDenied = "KMSKeyAccessDenied"
+    KMSKeyNotFound = "KMSKeyNotFound"
+    InvalidStateKMSKey = "InvalidStateKMSKey"
+    DisabledKMSKey = "DisabledKMSKey"
+    EFSIOError = "EFSIOError"
+    EFSMountConnectivityError = "EFSMountConnectivityError"
+    EFSMountFailure = "EFSMountFailure"
+    EFSMountTimeout = "EFSMountTimeout"
+    InvalidRuntime = "InvalidRuntime"
+    InvalidZipFileException = "InvalidZipFileException"
+    FunctionError = "FunctionError"
 
 
 class LogType(str):
@@ -201,6 +212,17 @@ class Runtime(str):
     ruby2_7 = "ruby2.7"
     provided = "provided"
     provided_al2 = "provided.al2"
+    nodejs18_x = "nodejs18.x"
+
+
+class SnapStartApplyOn(str):
+    PublishedVersions = "PublishedVersions"
+    None_ = "None"
+
+
+class SnapStartOptimizationStatus(str):
+    On = "On"
+    Off = "Off"
 
 
 class SourceAccessType(str):
@@ -235,6 +257,17 @@ class StateReasonCode(str):
     ImageDeleted = "ImageDeleted"
     ImageAccessDenied = "ImageAccessDenied"
     InvalidImage = "InvalidImage"
+    KMSKeyAccessDenied = "KMSKeyAccessDenied"
+    KMSKeyNotFound = "KMSKeyNotFound"
+    InvalidStateKMSKey = "InvalidStateKMSKey"
+    DisabledKMSKey = "DisabledKMSKey"
+    EFSIOError = "EFSIOError"
+    EFSMountConnectivityError = "EFSMountConnectivityError"
+    EFSMountFailure = "EFSMountFailure"
+    EFSMountTimeout = "EFSMountTimeout"
+    InvalidRuntime = "InvalidRuntime"
+    InvalidZipFileException = "InvalidZipFileException"
+    FunctionError = "FunctionError"
 
 
 class ThrottleReason(str):
@@ -245,6 +278,7 @@ class ThrottleReason(str):
     )
     ReservedFunctionInvocationRateLimitExceeded = "ReservedFunctionInvocationRateLimitExceeded"
     CallerRateLimitExceeded = "CallerRateLimitExceeded"
+    ConcurrentSnapshotCreateLimitExceeded = "ConcurrentSnapshotCreateLimitExceeded"
 
 
 class TracingMode(str):
@@ -470,6 +504,27 @@ class ServiceException(ServiceException):
     Type: Optional[String]
 
 
+class SnapStartException(ServiceException):
+    code: str = "SnapStartException"
+    sender_fault: bool = False
+    status_code: int = 400
+    Type: Optional[String]
+
+
+class SnapStartNotReadyException(ServiceException):
+    code: str = "SnapStartNotReadyException"
+    sender_fault: bool = False
+    status_code: int = 409
+    Type: Optional[String]
+
+
+class SnapStartTimeoutException(ServiceException):
+    code: str = "SnapStartTimeoutException"
+    sender_fault: bool = False
+    status_code: int = 408
+    Type: Optional[String]
+
+
 class SubnetIPAddressLimitReachedException(ServiceException):
     code: str = "SubnetIPAddressLimitReachedException"
     sender_fault: bool = False
@@ -571,6 +626,10 @@ class AllowedPublishers(TypedDict, total=False):
     SigningProfileVersionArns: SigningProfileVersionArns
 
 
+class AmazonManagedKafkaEventSourceConfig(TypedDict, total=False):
+    ConsumerGroupId: Optional[URI]
+
+
 ArchitecturesList = List[Architecture]
 Blob = bytes
 BlobStream = bytes
@@ -628,6 +687,10 @@ class CreateCodeSigningConfigResponse(TypedDict, total=False):
     CodeSigningConfig: CodeSigningConfig
 
 
+class SelfManagedKafkaEventSourceConfig(TypedDict, total=False):
+    ConsumerGroupId: Optional[URI]
+
+
 FunctionResponseTypeList = List[FunctionResponseType]
 EndpointLists = List[Endpoint]
 Endpoints = Dict[EndPointType, EndpointLists]
@@ -694,6 +757,12 @@ class CreateEventSourceMappingRequest(ServiceRequest):
     SourceAccessConfigurations: Optional[SourceAccessConfigurations]
     SelfManagedEventSource: Optional[SelfManagedEventSource]
     FunctionResponseTypes: Optional[FunctionResponseTypeList]
+    AmazonManagedKafkaEventSourceConfig: Optional[AmazonManagedKafkaEventSourceConfig]
+    SelfManagedKafkaEventSourceConfig: Optional[SelfManagedKafkaEventSourceConfig]
+
+
+class SnapStart(TypedDict, total=False):
+    ApplyOn: Optional[SnapStartApplyOn]
 
 
 class EphemeralStorage(TypedDict, total=False):
@@ -774,6 +843,7 @@ class CreateFunctionRequest(ServiceRequest):
     CodeSigningConfigArn: Optional[CodeSigningConfigArn]
     Architectures: Optional[ArchitecturesList]
     EphemeralStorage: Optional[EphemeralStorage]
+    SnapStart: Optional[SnapStart]
 
 
 class CreateFunctionUrlConfigRequest(ServiceRequest):
@@ -875,6 +945,8 @@ class EventSourceMappingConfiguration(TypedDict, total=False):
     MaximumRetryAttempts: Optional[MaximumRetryAttemptsEventSourceMapping]
     TumblingWindowInSeconds: Optional[TumblingWindowInSeconds]
     FunctionResponseTypes: Optional[FunctionResponseTypeList]
+    AmazonManagedKafkaEventSourceConfig: Optional[AmazonManagedKafkaEventSourceConfig]
+    SelfManagedKafkaEventSourceConfig: Optional[SelfManagedKafkaEventSourceConfig]
 
 
 EventSourceMappingsList = List[EventSourceMappingConfiguration]
@@ -888,6 +960,11 @@ class FunctionCodeLocation(TypedDict, total=False):
     ResolvedImageUri: Optional[String]
 
 
+class SnapStartResponse(TypedDict, total=False):
+    ApplyOn: Optional[SnapStartApplyOn]
+    OptimizationStatus: Optional[SnapStartOptimizationStatus]
+
+
 class ImageConfigError(TypedDict, total=False):
     ErrorCode: Optional[String]
     Message: Optional[SensitiveString]
@@ -952,6 +1029,7 @@ class FunctionConfiguration(TypedDict, total=False):
     SigningJobArn: Optional[Arn]
     Architectures: Optional[ArchitecturesList]
     EphemeralStorage: Optional[EphemeralStorage]
+    SnapStart: Optional[SnapStartResponse]
 
 
 class FunctionEventInvokeConfig(TypedDict, total=False):
@@ -1483,6 +1561,7 @@ class UpdateFunctionConfigurationRequest(ServiceRequest):
     FileSystemConfigs: Optional[FileSystemConfigList]
     ImageConfig: Optional[ImageConfig]
     EphemeralStorage: Optional[EphemeralStorage]
+    SnapStart: Optional[SnapStart]
 
 
 class UpdateFunctionEventInvokeConfigRequest(ServiceRequest):
@@ -1591,6 +1670,8 @@ class LambdaApi:
         source_access_configurations: SourceAccessConfigurations = None,
         self_managed_event_source: SelfManagedEventSource = None,
         function_response_types: FunctionResponseTypeList = None,
+        amazon_managed_kafka_event_source_config: AmazonManagedKafkaEventSourceConfig = None,
+        self_managed_kafka_event_source_config: SelfManagedKafkaEventSourceConfig = None,
     ) -> EventSourceMappingConfiguration:
         raise NotImplementedError
 
@@ -1620,6 +1701,7 @@ class LambdaApi:
         code_signing_config_arn: CodeSigningConfigArn = None,
         architectures: ArchitecturesList = None,
         ephemeral_storage: EphemeralStorage = None,
+        snap_start: SnapStart = None,
     ) -> FunctionConfiguration:
         raise NotImplementedError
 
@@ -2117,6 +2199,7 @@ class LambdaApi:
         file_system_configs: FileSystemConfigList = None,
         image_config: ImageConfig = None,
         ephemeral_storage: EphemeralStorage = None,
+        snap_start: SnapStart = None,
     ) -> FunctionConfiguration:
         raise NotImplementedError
 
