@@ -18,6 +18,9 @@ Identity = str
 IncludeExecutionData = bool
 IncludeExecutionDataGetExecutionHistory = bool
 ListExecutionsPageToken = str
+LongArn = str
+MapRunLabel = str
+MaxConcurrency = int
 Name = str
 PageSize = int
 PageToken = str
@@ -29,6 +32,7 @@ SensitiveError = str
 TagKey = str
 TagValue = str
 TaskToken = str
+ToleratedFailurePercentage = float
 TraceHeader = str
 UnsignedInteger = int
 includedDetails = bool
@@ -99,6 +103,10 @@ class HistoryEventType(str):
     WaitStateAborted = "WaitStateAborted"
     WaitStateEntered = "WaitStateEntered"
     WaitStateExited = "WaitStateExited"
+    MapRunAborted = "MapRunAborted"
+    MapRunFailed = "MapRunFailed"
+    MapRunStarted = "MapRunStarted"
+    MapRunSucceeded = "MapRunSucceeded"
 
 
 class LogLevel(str):
@@ -108,6 +116,13 @@ class LogLevel(str):
     OFF = "OFF"
 
 
+class MapRunStatus(str):
+    RUNNING = "RUNNING"
+    SUCCEEDED = "SUCCEEDED"
+    FAILED = "FAILED"
+    ABORTED = "ABORTED"
+
+
 class StateMachineStatus(str):
     ACTIVE = "ACTIVE"
     DELETING = "DELETING"
@@ -124,6 +139,12 @@ class SyncExecutionStatus(str):
     TIMED_OUT = "TIMED_OUT"
 
 
+class ValidationExceptionReason(str):
+    API_DOES_NOT_SUPPORT_LABELED_ARNS = "API_DOES_NOT_SUPPORT_LABELED_ARNS"
+    MISSING_REQUIRED_PARAMETER = "MISSING_REQUIRED_PARAMETER"
+    CANNOT_UPDATE_COMPLETED_MAP_RUN = "CANNOT_UPDATE_COMPLETED_MAP_RUN"
+
+
 class ActivityDoesNotExist(ServiceException):
     code: str = "ActivityDoesNotExist"
     sender_fault: bool = False
@@ -270,6 +291,13 @@ class TooManyTags(ServiceException):
     resourceName: Optional[Arn]
 
 
+class ValidationException(ServiceException):
+    code: str = "ValidationException"
+    sender_fault: bool = False
+    status_code: int = 400
+    reason: Optional[ValidationExceptionReason]
+
+
 class ActivityFailedEventDetails(TypedDict, total=False):
     error: Optional[SensitiveError]
     cause: Optional[SensitiveCause]
@@ -435,6 +463,54 @@ class DescribeExecutionOutput(TypedDict, total=False):
     output: Optional[SensitiveData]
     outputDetails: Optional[CloudWatchEventsExecutionDataDetails]
     traceHeader: Optional[TraceHeader]
+    mapRunArn: Optional[LongArn]
+    error: Optional[SensitiveError]
+    cause: Optional[SensitiveCause]
+
+
+class DescribeMapRunInput(ServiceRequest):
+    mapRunArn: LongArn
+
+
+UnsignedLong = int
+
+
+class MapRunExecutionCounts(TypedDict, total=False):
+    pending: UnsignedLong
+    running: UnsignedLong
+    succeeded: UnsignedLong
+    failed: UnsignedLong
+    timedOut: UnsignedLong
+    aborted: UnsignedLong
+    total: UnsignedLong
+    resultsWritten: UnsignedLong
+
+
+class MapRunItemCounts(TypedDict, total=False):
+    pending: UnsignedLong
+    running: UnsignedLong
+    succeeded: UnsignedLong
+    failed: UnsignedLong
+    timedOut: UnsignedLong
+    aborted: UnsignedLong
+    total: UnsignedLong
+    resultsWritten: UnsignedLong
+
+
+ToleratedFailureCount = int
+
+
+class DescribeMapRunOutput(TypedDict, total=False):
+    mapRunArn: LongArn
+    executionArn: Arn
+    status: MapRunStatus
+    startDate: Timestamp
+    stopDate: Optional[Timestamp]
+    maxConcurrency: MaxConcurrency
+    toleratedFailurePercentage: ToleratedFailurePercentage
+    toleratedFailureCount: ToleratedFailureCount
+    itemCounts: MapRunItemCounts
+    executionCounts: MapRunExecutionCounts
 
 
 class DescribeStateMachineForExecutionInput(ServiceRequest):
@@ -449,6 +525,8 @@ class DescribeStateMachineForExecutionOutput(TypedDict, total=False):
     updateDate: Timestamp
     loggingConfiguration: Optional[LoggingConfiguration]
     tracingConfiguration: Optional[TracingConfiguration]
+    mapRunArn: Optional[LongArn]
+    label: Optional[MapRunLabel]
 
 
 class DescribeStateMachineInput(ServiceRequest):
@@ -467,6 +545,7 @@ DescribeStateMachineOutput = TypedDict(
         "creationDate": Timestamp,
         "loggingConfiguration": Optional[LoggingConfiguration],
         "tracingConfiguration": Optional[TracingConfiguration],
+        "label": Optional[MapRunLabel],
     },
     total=False,
 )
@@ -490,6 +569,8 @@ class ExecutionListItem(TypedDict, total=False):
     status: ExecutionStatus
     startDate: Timestamp
     stopDate: Optional[Timestamp]
+    mapRunArn: Optional[LongArn]
+    itemCount: Optional[UnsignedInteger]
 
 
 ExecutionList = List[ExecutionListItem]
@@ -529,6 +610,15 @@ class GetExecutionHistoryInput(ServiceRequest):
     includeExecutionData: Optional[IncludeExecutionDataGetExecutionHistory]
 
 
+class MapRunFailedEventDetails(TypedDict, total=False):
+    error: Optional[SensitiveError]
+    cause: Optional[SensitiveCause]
+
+
+class MapRunStartedEventDetails(TypedDict, total=False):
+    mapRunArn: Optional[LongArn]
+
+
 class StateExitedEventDetails(TypedDict, total=False):
     name: Name
     output: Optional[SensitiveData]
@@ -556,11 +646,16 @@ class LambdaFunctionStartFailedEventDetails(TypedDict, total=False):
     cause: Optional[SensitiveCause]
 
 
+class TaskCredentials(TypedDict, total=False):
+    roleArn: Optional[LongArn]
+
+
 class LambdaFunctionScheduledEventDetails(TypedDict, total=False):
     resource: Arn
     input: Optional[SensitiveData]
     inputDetails: Optional[HistoryEventExecutionDataDetails]
     timeoutInSeconds: Optional[TimeoutInSeconds]
+    taskCredentials: Optional[TaskCredentials]
 
 
 class LambdaFunctionScheduleFailedEventDetails(TypedDict, total=False):
@@ -629,6 +724,7 @@ class TaskScheduledEventDetails(TypedDict, total=False):
     parameters: ConnectorParameters
     timeoutInSeconds: Optional[TimeoutInSeconds]
     heartbeatInSeconds: Optional[TimeoutInSeconds]
+    taskCredentials: Optional[TaskCredentials]
 
 
 class TaskFailedEventDetails(TypedDict, total=False):
@@ -679,6 +775,8 @@ HistoryEvent = TypedDict(
         "lambdaFunctionTimedOutEventDetails": Optional[LambdaFunctionTimedOutEventDetails],
         "stateEnteredEventDetails": Optional[StateEnteredEventDetails],
         "stateExitedEventDetails": Optional[StateExitedEventDetails],
+        "mapRunStartedEventDetails": Optional[MapRunStartedEventDetails],
+        "mapRunFailedEventDetails": Optional[MapRunFailedEventDetails],
     },
     total=False,
 )
@@ -701,10 +799,11 @@ class ListActivitiesOutput(TypedDict, total=False):
 
 
 class ListExecutionsInput(ServiceRequest):
-    stateMachineArn: Arn
+    stateMachineArn: Optional[Arn]
     statusFilter: Optional[ExecutionStatus]
     maxResults: Optional[PageSize]
     nextToken: Optional[ListExecutionsPageToken]
+    mapRunArn: Optional[LongArn]
 
 
 class ListExecutionsOutput(TypedDict, total=False):
@@ -712,6 +811,28 @@ class ListExecutionsOutput(TypedDict, total=False):
     nextToken: Optional[ListExecutionsPageToken]
 
 
+class ListMapRunsInput(ServiceRequest):
+    executionArn: Arn
+    maxResults: Optional[PageSize]
+    nextToken: Optional[PageToken]
+
+
+class MapRunListItem(TypedDict, total=False):
+    executionArn: Arn
+    mapRunArn: LongArn
+    stateMachineArn: Arn
+    startDate: Timestamp
+    stopDate: Optional[Timestamp]
+
+
+MapRunList = List[MapRunListItem]
+
+
+class ListMapRunsOutput(TypedDict, total=False):
+    mapRuns: MapRunList
+    nextToken: Optional[PageToken]
+
+
 class ListStateMachinesInput(ServiceRequest):
     maxResults: Optional[PageSize]
     nextToken: Optional[PageToken]
@@ -837,6 +958,17 @@ class UntagResourceOutput(TypedDict, total=False):
     pass
 
 
+class UpdateMapRunInput(ServiceRequest):
+    mapRunArn: LongArn
+    maxConcurrency: Optional[MaxConcurrency]
+    toleratedFailurePercentage: Optional[ToleratedFailurePercentage]
+    toleratedFailureCount: Optional[ToleratedFailureCount]
+
+
+class UpdateMapRunOutput(TypedDict, total=False):
+    pass
+
+
 class UpdateStateMachineInput(ServiceRequest):
     stateMachineArn: Arn
     definition: Optional[Definition]
@@ -888,6 +1020,12 @@ class StepfunctionsApi:
     ) -> DescribeExecutionOutput:
         raise NotImplementedError
 
+    @handler("DescribeMapRun")
+    def describe_map_run(
+        self, context: RequestContext, map_run_arn: LongArn
+    ) -> DescribeMapRunOutput:
+        raise NotImplementedError
+
     @handler("DescribeStateMachine")
     def describe_state_machine(
         self, context: RequestContext, state_machine_arn: Arn
@@ -928,13 +1066,24 @@ class StepfunctionsApi:
     def list_executions(
         self,
         context: RequestContext,
-        state_machine_arn: Arn,
+        state_machine_arn: Arn = None,
         status_filter: ExecutionStatus = None,
         max_results: PageSize = None,
         next_token: ListExecutionsPageToken = None,
+        map_run_arn: LongArn = None,
     ) -> ListExecutionsOutput:
         raise NotImplementedError
 
+    @handler("ListMapRuns")
+    def list_map_runs(
+        self,
+        context: RequestContext,
+        execution_arn: Arn,
+        max_results: PageSize = None,
+        next_token: PageToken = None,
+    ) -> ListMapRunsOutput:
+        raise NotImplementedError
+
     @handler("ListStateMachines")
     def list_state_machines(
         self, context: RequestContext, max_results: PageSize = None, next_token: PageToken = None
@@ -1013,6 +1162,17 @@ class StepfunctionsApi:
     ) -> UntagResourceOutput:
         raise NotImplementedError
 
+    @handler("UpdateMapRun")
+    def update_map_run(
+        self,
+        context: RequestContext,
+        map_run_arn: LongArn,
+        max_concurrency: MaxConcurrency = None,
+        tolerated_failure_percentage: ToleratedFailurePercentage = None,
+        tolerated_failure_count: ToleratedFailureCount = None,
+    ) -> UpdateMapRunOutput:
+        raise NotImplementedError
+
     @handler("UpdateStateMachine")
     def update_state_machine(
         self,
