@@ -53,6 +53,8 @@ class CLMLanguageCode(str):
     es_US = "es-US"
     en_GB = "en-GB"
     en_AU = "en-AU"
+    de_DE = "de-DE"
+    ja_JP = "ja-JP"
 
 
 class CallAnalyticsJobStatus(str):
@@ -62,6 +64,11 @@ class CallAnalyticsJobStatus(str):
     COMPLETED = "COMPLETED"
 
 
+class InputType(str):
+    REAL_TIME = "REAL_TIME"
+    POST_CALL = "POST_CALL"
+
+
 class LanguageCode(str):
     af_ZA = "af-ZA"
     ar_AE = "ar-AE"
@@ -379,6 +386,7 @@ class CategoryProperties(TypedDict, total=False):
     Rules: Optional[RuleList]
     CreateTime: Optional[DateTime]
     LastUpdateTime: Optional[DateTime]
+    InputType: Optional[InputType]
 
 
 CategoryPropertiesList = List[CategoryProperties]
@@ -387,6 +395,7 @@ CategoryPropertiesList = List[CategoryProperties]
 class CreateCallAnalyticsCategoryRequest(ServiceRequest):
     CategoryName: CategoryName
     Rules: RuleList
+    InputType: Optional[InputType]
 
 
 class CreateCallAnalyticsCategoryResponse(TypedDict, total=False):
@@ -958,6 +967,7 @@ class UntagResourceResponse(TypedDict, total=False):
 class UpdateCallAnalyticsCategoryRequest(ServiceRequest):
     CategoryName: CategoryName
     Rules: RuleList
+    InputType: Optional[InputType]
 
 
 class UpdateCallAnalyticsCategoryResponse(TypedDict, total=False):
@@ -1010,7 +1020,11 @@ class TranscribeApi:
 
     @handler("CreateCallAnalyticsCategory")
     def create_call_analytics_category(
-        self, context: RequestContext, category_name: CategoryName, rules: RuleList
+        self,
+        context: RequestContext,
+        category_name: CategoryName,
+        rules: RuleList,
+        input_type: InputType = None,
     ) -> CreateCallAnalyticsCategoryResponse:
         raise NotImplementedError
 
@@ -1301,7 +1315,11 @@ class TranscribeApi:
 
     @handler("UpdateCallAnalyticsCategory")
     def update_call_analytics_category(
-        self, context: RequestContext, category_name: CategoryName, rules: RuleList
+        self,
+        context: RequestContext,
+        category_name: CategoryName,
+        rules: RuleList,
+        input_type: InputType = None,
     ) -> UpdateCallAnalyticsCategoryResponse:
         raise NotImplementedError
 
