@@ -4,6 +4,7 @@ from .. import chain
 from . import analytics, auth, codec, cors, fallback, internal, legacy, logging, region, service
 
 enforce_cors = cors.CorsEnforcer()
+preprocess_request = chain.CompositeHandler()
 add_cors_response_headers = cors.CorsResponseEnricher()
 content_decoder = codec.ContentDecoder()
 parse_service_name = service.ServiceNameParser()
@@ -19,6 +20,7 @@ handle_internal_failure = fallback.InternalFailureHandler()
 serve_custom_service_request_handlers = chain.CompositeHandler()
 serve_localstack_resources = internal.LocalstackResourceHandler()
 run_custom_response_handlers = chain.CompositeResponseHandler()
+modify_service_response = service.ServiceResponseHandlers()
 parse_service_response = service.ServiceResponseParser()
 # legacy compatibility handlers
 serve_edge_router_rules = legacy.EdgeRouterHandler()
