@@ -6,7 +6,11 @@ from typing import Any
 
 from botocore.model import ServiceModel
 
-from localstack.aws.accounts import get_account_id_from_access_key_id, set_aws_access_key_id
+from localstack.aws.accounts import (
+    get_account_id_from_access_key_id,
+    set_aws_access_key_id,
+    set_aws_account_id,
+)
 from localstack.aws.api import RequestContext
 from localstack.aws.skeleton import Skeleton
 from localstack.aws.spec import load_service
@@ -28,7 +32,11 @@ def get_account_id_from_request(request: Request) -> str:
         extract_access_key_id_from_auth_header(request.headers) or TEST_AWS_ACCESS_KEY_ID
     )
     set_aws_access_key_id(access_key_id)
-    return get_account_id_from_access_key_id(access_key_id)
+
+    account_id = get_account_id_from_access_key_id(access_key_id)
+    set_aws_account_id(account_id)
+
+    return account_id
 
 
 class AwsApiListener(ProxyListenerAdapter):
