@@ -4,7 +4,6 @@ import os
 from typing import Optional, Union
 
 from localstack import config
-from localstack.services import install
 from localstack.services.awslambda.lambda_executors import (
     AdditionalInvocationOptions,
     InvocationContext,
@@ -20,13 +19,13 @@ from localstack.services.awslambda.lambda_utils import (
     LAMBDA_RUNTIME_NODEJS12X,
     LAMBDA_RUNTIME_NODEJS14X,
     LAMBDA_RUNTIME_NODEJS16X,
-    LAMBDA_RUNTIME_PYTHON36,
     LAMBDA_RUNTIME_PYTHON37,
     LAMBDA_RUNTIME_PYTHON38,
     get_executor_mode,
     is_python_runtime,
 )
 from localstack.utils import common
+from localstack.utils.http import download
 
 # logger
 LOG = logging.getLogger(__name__)
@@ -60,6 +59,10 @@ THUNDRA_PYTHON_AGENT_LOCAL_PATH_ON_HOST: Optional[str] = None
 ################
 
 
+def _log_install_msg(component):
+    LOG.info("Downloading and installing %s. This may take some time.", component)
+
+
 def _get_apikey(env_vars):
     thundra_apikey = env_vars.get(THUNDRA_APIKEY_ENV_VAR_NAME)
 
@@ -119,11 +122,12 @@ def _init_java_agent_configs():
     THUNDRA_JAVA_AGENT_LOCAL_PATH = "%s/%s" % (config.dirs.tmp, jar_name)
 
 
+# TODO migrate these install commands to package installers
 def _install_java_agent():
     # Install Thundra Java agent JAR file
     if not os.path.exists(THUNDRA_JAVA_AGENT_LOCAL_PATH):
-        install.log_install_msg("Thundra Java agent", verbatim=True)
-        install.download(THUNDRA_JAVA_AGENT_REMOTE_URL, THUNDRA_JAVA_AGENT_LOCAL_PATH)
+        _log_install_msg("Thundra Java agent")
+        download(THUNDRA_JAVA_AGENT_REMOTE_URL, THUNDRA_JAVA_AGENT_LOCAL_PATH)
 
 
 def _is_java8_lambda(func_details):
@@ -232,7 +236,7 @@ def _init_node_agent_configs() -> bool:
 def _install_node_agent() -> bool:
     # Install Thundra Node agent NPM package
     if not os.path.exists(THUNDRA_NODE_AGENT_LOCAL_PATH):
-        install.log_install_msg("Thundra Node agent", verbatim=True)
+        _log_install_msg("Thundra Node agent")
         try:
             install_thundra_cmd = "npm install --prefix %s @thundra/core@%s --no-save" % (
                 THUNDRA_NODE_AGENT_LOCAL_PATH,
@@ -342,7 +346,7 @@ def _init_python_agent_configs() -> bool:
 def _install_python_agent() -> bool:
     # Install Thundra Python agent PIP package
     if not os.path.exists(THUNDRA_PYTHON_AGENT_LOCAL_PATH):
-        install.log_install_msg("Thundra Python agent", verbatim=True)
+        _log_install_msg("Thundra Python agent")
         try:
             install_thundra_cmd = "pip install --target=%s thundra==%s --no-warn-conflicts" % (
                 THUNDRA_PYTHON_AGENT_LOCAL_PATH,
@@ -358,7 +362,6 @@ def _install_python_agent() -> bool:
 def _is_python_lambda_with_support_version(func_details):
     runtime = getattr(func_details, "runtime", func_details)
     return runtime in [
-        LAMBDA_RUNTIME_PYTHON36,
         LAMBDA_RUNTIME_PYTHON37,
         LAMBDA_RUNTIME_PYTHON38,
     ]
