@@ -5,13 +5,12 @@ from typing import Any, Dict, Union
 from urllib.parse import urljoin
 
 import requests
-from flask import Response as FlaskResponse
 from jsonschema import ValidationError, validate
 from requests.models import Response
 
 from localstack import config
 from localstack.aws.accounts import get_aws_account_id
-from localstack.constants import APPLICATION_JSON, HEADER_LOCALSTACK_AUTHORIZATION
+from localstack.constants import APPLICATION_JSON
 from localstack.services.apigateway import helpers
 from localstack.services.apigateway.context import ApiInvocationContext
 from localstack.services.apigateway.helpers import (
@@ -21,25 +20,20 @@ from localstack.services.apigateway.helpers import (
     make_error_response,
 )
 from localstack.services.apigateway.integration import (
+    LambdaIntegration,
     LambdaProxyIntegration,
     MockIntegration,
+    SnsIntegration,
+    StepFunctionIntegration,
+)
+from localstack.services.apigateway.templates import (
     RequestTemplates,
     ResponseTemplates,
-    SnsIntegration,
     VtlTemplate,
 )
-from localstack.services.awslambda import lambda_api
-from localstack.services.kinesis import kinesis_listener
-from localstack.services.stepfunctions.stepfunctions_utils import await_sfn_execution_result
 from localstack.utils import common
 from localstack.utils.aws import aws_stack
-from localstack.utils.aws.aws_responses import (
-    LambdaResponse,
-    flask_to_requests_response,
-    request_response_stream,
-    requests_response,
-)
-from localstack.utils.common import camel_to_snake_case, json_safe
+from localstack.utils.aws.aws_responses import request_response_stream, requests_response
 
 # set up logger
 from localstack.utils.http import add_query_params_to_url
@@ -313,8 +307,6 @@ def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext
     method = invocation_context.method
     data = invocation_context.data
     headers = invocation_context.headers
-    api_id = invocation_context.api_id
-    stage = invocation_context.stage
     resource_path = invocation_context.resource_path
     integration = invocation_context.integration
     integration_response = integration.get("integrationResponses", {})
@@ -324,10 +316,6 @@ def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext
     integration_type_orig = integration.get("type") or integration.get("integrationType") or ""
     integration_type = integration_type_orig.upper()
     uri = integration.get("uri") or integration.get("integrationUri") or ""
-    # XXX we need replace the internal Authorization header with an Authorization header set from
-    # the customer, even if it's empty that's what's expected in the integration.
-    custom_auth_header = invocation_context.headers.pop(HEADER_LOCALSTACK_AUTHORIZATION, "")
-    invocation_context.headers["Authorization"] = custom_auth_header
 
     try:
         path_params = extract_path_params(path=relative_path, extracted_path=resource_path)
@@ -341,67 +329,7 @@ def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext
         if integration_type == "AWS_PROXY":
             return LambdaProxyIntegration().invoke(invocation_context)
         elif integration_type == "AWS":
-            func_arn = uri
-            if ":lambda:path" in uri:
-                func_arn = (
-                    uri.split(":lambda:path")[1].split("functions/")[1].split("/invocations")[0]
-                )
-
-            headers = helpers.create_invocation_headers(invocation_context)
-            invocation_context.context = helpers.get_event_request_context(invocation_context)
-            invocation_context.stage_variables = helpers.get_stage_variables(invocation_context)
-            if invocation_context.authorizer_type:
-                invocation_context.context["authorizer"] = invocation_context.auth_context
-
-            request_templates = RequestTemplates()
-            payload = request_templates.render(invocation_context)
-
-            # TODO: change this signature to InvocationContext as well!
-            result = lambda_api.process_apigateway_invocation(
-                func_arn,
-                relative_path,
-                payload,
-                stage,
-                api_id,
-                headers,
-                is_base64_encoded=invocation_context.is_data_base64_encoded,
-                path_params=path_params,
-                query_string_params=query_string_params,
-                method=method,
-                resource_path=resource_path,
-                request_context=invocation_context.context,
-                stage_variables=invocation_context.stage_variables,
-            )
-
-            if isinstance(result, FlaskResponse):
-                response = flask_to_requests_response(result)
-            elif isinstance(result, Response):
-                response = result
-            else:
-                response = LambdaResponse()
-
-                is_async = headers.get("X-Amz-Invocation-Type", "").strip("'") == "Event"
-
-                if is_async:
-                    response._content = ""
-                    response.status_code = 200
-                else:
-                    parsed_result = (
-                        result if isinstance(result, dict) else json.loads(str(result or "{}"))
-                    )
-                    parsed_result = common.json_safe(parsed_result)
-                    parsed_result = {} if parsed_result is None else parsed_result
-                    response.status_code = 200
-                    response._content = parsed_result
-                    update_content_length(response)
-
-            # apply custom response template
-            invocation_context.response = response
-
-            response_templates = ResponseTemplates()
-            response_templates.render(invocation_context)
-            invocation_context.response.headers["Content-Length"] = str(len(response.content or ""))
-            return invocation_context.response
+            return LambdaIntegration().invoke(invocation_context)
 
         raise Exception(
             f'API Gateway integration type "{integration_type}", action "{uri}", method "{method}"'
@@ -410,11 +338,11 @@ def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext
     elif integration_type == "AWS":
         if "kinesis:action/" in uri:
             if uri.endswith("kinesis:action/PutRecord"):
-                target = kinesis_listener.ACTION_PUT_RECORD
+                target = "Kinesis_20131202.PutRecord"
             elif uri.endswith("kinesis:action/PutRecords"):
-                target = kinesis_listener.ACTION_PUT_RECORDS
+                target = "Kinesis_20131202.PutRecords"
             elif uri.endswith("kinesis:action/ListStreams"):
-                target = kinesis_listener.ACTION_LIST_STREAMS
+                target = "Kinesis_20131202.ListStreams"
             else:
                 LOG.info(
                     f"Unexpected API Gateway integration URI '{uri}' for integration type {integration_type}",
@@ -448,60 +376,7 @@ def invoke_rest_api_integration_backend(invocation_context: ApiInvocationContext
             return invocation_context.response
 
         elif "states:action/" in uri:
-            action = uri.split("/")[-1]
-
-            if APPLICATION_JSON in integration.get("requestTemplates", {}):
-                request_templates = RequestTemplates()
-                payload = request_templates.render(invocation_context)
-                payload = json.loads(payload)
-            else:
-                # XXX decoding in py3 sounds wrong, this actually might break
-                payload = json.loads(data.decode("utf-8"))
-            client = aws_stack.connect_to_service("stepfunctions")
-
-            if isinstance(payload.get("input"), dict):
-                payload["input"] = json.dumps(payload["input"])
-
-            # Hot fix since step functions local package responses: Unsupported Operation: 'StartSyncExecution'
-            method_name = (
-                camel_to_snake_case(action) if action != "StartSyncExecution" else "start_execution"
-            )
-
-            try:
-                method = getattr(client, method_name)
-            except AttributeError:
-                msg = "Invalid step function action: %s" % method_name
-                LOG.error(msg)
-                return make_error_response(msg, 400)
-
-            result = method(**payload)
-            result = json_safe({k: result[k] for k in result if k not in "ResponseMetadata"})
-            response = requests_response(
-                content=result,
-                headers=aws_stack.mock_aws_request_headers(),
-            )
-
-            if action == "StartSyncExecution":
-                # poll for the execution result and return it
-                result = await_sfn_execution_result(result["executionArn"])
-                result_status = result.get("status")
-                if result_status != "SUCCEEDED":
-                    return make_error_response(
-                        "StepFunctions execution %s failed with status '%s'"
-                        % (result["executionArn"], result_status),
-                        500,
-                    )
-                result = json_safe(result)
-                response = requests_response(content=result)
-
-            # apply response templates
-            invocation_context.response = response
-            response_templates = ResponseTemplates()
-            response_templates.render(invocation_context)
-            # response = apply_request_response_templates(
-            #     response, response_templates, content_type=APPLICATION_JSON
-            # )
-            return response
+            return StepFunctionIntegration().invoke(invocation_context)
         # https://docs.aws.amazon.com/apigateway/api-reference/resource/integration/
         elif ("s3:path/" in uri or "s3:action/" in uri) and method == "GET":
             s3 = aws_stack.connect_to_service("s3")
