@@ -5,7 +5,6 @@ from localstack.aws.accounts import get_aws_account_id
 from localstack.services.awslambda.event_source_listeners.stream_event_source_listener import (
     StreamEventSourceListener,
 )
-from localstack.services.awslambda.lambda_api import get_event_sources
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import first_char_to_lower, to_str
 from localstack.utils.threads import FuncThread
@@ -25,7 +24,7 @@ class KinesisEventSourceListener(StreamEventSourceListener):
         return "kinesis"
 
     def _get_matching_event_sources(self) -> List[Dict]:
-        event_sources = get_event_sources(source_arn=r".*:kinesis:.*")
+        event_sources = self._invoke_adapter.get_event_sources(source_arn=r".*:kinesis:.*")
         return [source for source in event_sources if source["State"] == "Enabled"]
 
     def _get_stream_client(self, region_name):
