@@ -1,5 +1,7 @@
 import json
 import logging
+import random
+import string
 
 from botocore.exceptions import ClientError
 
@@ -15,7 +17,7 @@ from localstack.services.cloudformation.deployment_utils import (
 )
 from localstack.services.cloudformation.service_models import GenericBaseModel
 from localstack.services.iam.provider import SERVICE_LINKED_ROLE_PATH_PREFIX
-from localstack.utils.aws import aws_stack
+from localstack.utils.aws import arns, aws_stack
 from localstack.utils.common import ensure_list
 from localstack.utils.functions import call_safe
 
@@ -28,7 +30,7 @@ class IAMManagedPolicy(GenericBaseModel):
         return "AWS::IAM::ManagedPolicy"
 
     def get_physical_resource_id(self, attribute=None, **kwargs):
-        return aws_stack.policy_arn(self.props["ManagedPolicyName"])
+        return arns.policy_arn(self.props["ManagedPolicyName"])
 
     def fetch_state(self, stack_name, resources):
         return IAMPolicy.get_policy_state(self, stack_name, resources, managed_policy=True)
@@ -230,7 +232,7 @@ class IAMRole(GenericBaseModel):
         if not role_name:
             return role_name
         if attribute == "Arn":
-            return aws_stack.role_arn(role_name)
+            return arns.role_arn(role_name)
         return role_name
 
     def fetch_state(self, stack_name, resources):
@@ -439,13 +441,15 @@ class IAMPolicy(GenericBaseModel):
     def fetch_state(self, stack_name, resources):
         return IAMPolicy.get_policy_state(self, stack_name, resources, managed_policy=False)
 
-    def get_physical_resource_id(self, attribute=None, **kwargs):
-        if attribute == "Arn":
-            return aws_stack.policy_arn(self.props.get("PolicyName"))
-        return self.props.get("PolicyName")
-
     @classmethod
     def get_deploy_templates(cls):
+        def _store_physical_id(result, resource_id, resources, resource_type):
+            resource = resources[resource_id]
+            # the physical resource ID here has a bit of a weird format
+            # e.g. 'stack-fnSe-1OKWZIBB89193' where fnSe are the first 4 characters of the LogicalResourceId (or name?)
+            suffix = "".join(random.choices(string.ascii_uppercase + string.digits, k=13))
+            resource["PhysicalResourceId"] = f"stack-{resource.get('PolicyName', '')[:4]}-{suffix}"
+
         def _create(resource_id, resources, resource_type, func, stack_name, *args, **kwargs):
             iam = aws_stack.connect_to_service("iam")
             props = resources[resource_id]["Properties"]
@@ -466,10 +470,13 @@ class IAMPolicy(GenericBaseModel):
                 )
 
         def _delete_params(params, *args, **kwargs):
-            return {"PolicyArn": aws_stack.policy_arn(params["PolicyName"])}
+            return {"PolicyArn": arns.policy_arn(params["PolicyName"])}
 
         return {
-            "create": {"function": _create},
+            "create": {
+                "function": _create,
+                "result_handler": _store_physical_id,
+            },
             "delete": {"function": "delete_policy", "parameters": _delete_params},
         }
 
@@ -487,7 +494,7 @@ class IAMPolicy(GenericBaseModel):
         users = props.get("Users", [])
         groups = props.get("Groups", [])
         if managed_policy:
-            result["policy"] = iam.get_policy(PolicyArn=aws_stack.policy_arn(policy_name))
+            result["policy"] = iam.get_policy(PolicyArn=arns.policy_arn(policy_name))
         for role in roles:
             role = obj.resolve_refs_recursively(stack_name, role, resources)
             policies = (
