@@ -5,6 +5,7 @@ from localstack.services.cloudformation.deployment_utils import (
 )
 from localstack.services.cloudformation.service_models import GenericBaseModel
 from localstack.utils.aws import aws_stack
+from localstack.utils.collections import select_attributes
 from localstack.utils.common import short_uid
 
 
@@ -27,6 +28,36 @@ class SSMParameter(GenericBaseModel):
         if not name:
             resource["Properties"]["Name"] = f"CFN-{resource['LogicalResourceId']}-{short_uid()}"
 
+    def update_resource(self, new_resource, stack_name, resources):
+        props = new_resource["Properties"]
+        parameters_to_select = [
+            "AllowedPattern",
+            "DataType",
+            "Description",
+            "Name",
+            "Policies",
+            "Tags",
+            "Tier",
+            "Type",
+            "Value",
+        ]
+        update_config_props = select_attributes(props, parameters_to_select)
+        update_config_props = self.resolve_refs_recursively(
+            stack_name, update_config_props, resources
+        )
+
+        if "Tags" in update_config_props:
+            update_config_props["Tags"] = [
+                {
+                    "Key": k,
+                    "Value": v,
+                }
+                for (k, v) in (update_config_props["Tags"] or {}).items()
+            ]
+
+        client = aws_stack.connect_to_service("ssm")
+        return client.put_parameter(Overwrite=True, **update_config_props)
+
     @staticmethod
     def get_deploy_templates():
         return {
