@@ -1,6 +1,7 @@
 import json
 import logging
 import re
+from collections import defaultdict
 from copy import deepcopy
 from typing import Any, Dict, List, Optional
 
@@ -65,7 +66,7 @@ from localstack.aws.api.cloudformation import (
     ValidateTemplateOutput,
 )
 from localstack.services.cloudformation.stores import CloudFormationStore, cloudformation_stores
-from localstack.utils.aws import aws_stack
+from localstack.utils.aws import arns, aws_stack
 from localstack.utils.cloudformation import template_deployer, template_preparer
 from localstack.utils.cloudformation.template_deployer import NoStackUpdates
 from localstack.utils.cloudformation.template_preparer import (
@@ -138,7 +139,7 @@ class Stack:
                 "LogicalResourceId"
             ] = (resource.get("LogicalResourceId") or resource_id)
         # initialize stack template attributes
-        stack_id = self.metadata.get("StackId") or aws_stack.cloudformation_stack_arn(
+        stack_id = self.metadata.get("StackId") or arns.cloudformation_stack_arn(
             self.stack_name, short_uid()
         )
         self.template["StackId"] = self.metadata["StackId"] = stack_id
@@ -474,9 +475,7 @@ class StackChangeSet(Stack):
 
         name = self.metadata["ChangeSetName"]
         if not self.metadata.get("ChangeSetId"):
-            self.metadata["ChangeSetId"] = aws_stack.cf_change_set_arn(
-                name, change_set_id=short_uid()
-            )
+            self.metadata["ChangeSetId"] = arns.cf_change_set_arn(name, change_set_id=short_uid())
 
         stack = self.stack = find_stack(self.metadata["StackName"])
         self.metadata["StackId"] = stack.stack_id
@@ -721,10 +720,10 @@ class CloudformationProvider(CloudformationApi):
             stack = Stack(request, template)
 
         result: GetTemplateSummaryOutput = stack.describe_details()
-        id_summaries = {}
+        result["Parameters"] = result.get("Parameters") or []
+        id_summaries = defaultdict(list)
         for resource_id, resource in stack.template_resources.items():
             res_type = resource["Type"]
-            id_summaries[res_type] = id_summaries.get(res_type) or []
             id_summaries[res_type].append(resource_id)
 
         result["ResourceTypes"] = list(id_summaries.keys())
@@ -732,6 +731,9 @@ class CloudformationProvider(CloudformationApi):
             {"ResourceType": key, "LogicalResourceIds": values}
             for key, values in id_summaries.items()
         ]
+        result["Version"] = stack.template.get("AWSTemplateFormatVersion", "2010-09-09")
+        # these do not appear in the output
+        result.pop("Capabilities", None)
         return result
 
     @handler("ValidateTemplate", expand=False)
@@ -1085,7 +1087,8 @@ class CloudformationProvider(CloudformationApi):
 
         # wait for completion of stack
         for stack in stacks_to_await:
-            aws_stack.await_stack_completion(stack[0], region_name=stack[1])
+            client = aws_stack.connect_to_service("cloudformation", region_name=stack[1])
+            client.get_waiter("stack_create_complete").wait(StackName=stack[0])
 
         # record operation
         operation = {
