@@ -13,6 +13,10 @@ from localstack.aws.api.cloudwatch import (
     CloudwatchApi,
     DescribeAlarmsInput,
     DescribeAlarmsOutput,
+    GetMetricDataInput,
+    GetMetricDataOutput,
+    GetMetricStatisticsInput,
+    GetMetricStatisticsOutput,
     ListTagsForResourceOutput,
     PutCompositeAlarmInput,
     PutMetricAlarmInput,
@@ -23,19 +27,21 @@ from localstack.aws.api.cloudwatch import (
     UntagResourceOutput,
 )
 from localstack.constants import DEFAULT_AWS_ACCOUNT_ID
+from localstack.deprecations import deprecated_endpoint
 from localstack.http import Request
 from localstack.services import moto
 from localstack.services.cloudwatch.alarm_scheduler import AlarmScheduler
 from localstack.services.edge import ROUTER
 from localstack.services.plugins import SERVICE_PLUGINS, ServiceLifecycleHook
-from localstack.utils.aws import aws_stack
+from localstack.utils.aws import arns, aws_stack
 from localstack.utils.aws.aws_stack import extract_access_key_id_from_auth_header
 from localstack.utils.patch import patch
 from localstack.utils.sync import poll_condition
 from localstack.utils.tagging import TaggingService
 from localstack.utils.threads import start_worker_thread
 
-PATH_GET_RAW_METRICS = "/cloudwatch/metrics/raw"
+PATH_GET_RAW_METRICS = "/_aws/cloudwatch/metrics/raw"
+DEPRECATED_PATH_GET_RAW_METRICS = "/cloudwatch/metrics/raw"
 MOTO_INITIAL_UNCHECKED_REASON = "Unchecked: Initial alarm creation"
 
 LOG = logging.getLogger(__name__)
@@ -61,7 +67,7 @@ def update_state(target, self, reason, reason_data, state_value):
     else:
         actions = self.insufficient_data_actions
     for action in actions:
-        data = aws_stack.parse_arn(action)
+        data = arns.parse_arn(action)
         # test for sns - can this be done in a more generic way?
         if data["service"] == "sns":
             service = aws_stack.connect_to_service(data["service"])
@@ -225,6 +231,16 @@ class CloudwatchProvider(CloudwatchApi, ServiceLifecycleHook):
 
     def on_after_init(self):
         ROUTER.add(PATH_GET_RAW_METRICS, self.get_raw_metrics)
+        # TODO remove with 2.0
+        ROUTER.add(
+            DEPRECATED_PATH_GET_RAW_METRICS,
+            deprecated_endpoint(
+                self.get_raw_metrics,
+                previous_path=DEPRECATED_PATH_GET_RAW_METRICS,
+                deprecation_version="1.3.0",
+                new_path=PATH_GET_RAW_METRICS,
+            ),
+        )
         self.alarm_scheduler = AlarmScheduler()
 
     def on_before_start(self):
@@ -241,7 +257,7 @@ class CloudwatchProvider(CloudwatchApi, ServiceLifecycleHook):
     def delete_alarms(self, context: RequestContext, alarm_names: AlarmNames) -> None:
         moto.call_moto(context)
         for alarm_name in alarm_names:
-            arn = aws_stack.cloudwatch_alarm_arn(alarm_name)
+            arn = arns.cloudwatch_alarm_arn(alarm_name)
             self.alarm_scheduler.delete_scheduler_for_alarm(arn)
 
     def get_raw_metrics(self, request: Request):
@@ -285,6 +301,26 @@ class CloudwatchProvider(CloudwatchApi, ServiceLifecycleHook):
         self.tags.tag_resource(resource_arn, tags)
         return TagResourceOutput()
 
+    @handler("GetMetricData", expand=False)
+    def get_metric_data(
+        self, context: RequestContext, request: GetMetricDataInput
+    ) -> GetMetricDataOutput:
+        result = moto.call_moto(context)
+        # moto currently uses hardcoded label metric_name + stat
+        # parity tests shows that default is MetricStat, but there might also be a label explicitly set
+        metric_data_queries = request["MetricDataQueries"]
+        for i in range(0, len(metric_data_queries)):
+            metric_query = metric_data_queries[i]
+            label = metric_query.get("Label") or metric_query.get("MetricStat", {}).get(
+                "Metric", {}
+            ).get("MetricName")
+            if label:
+                result["MetricDataResults"][i]["Label"] = label
+        if "Messages" not in result:
+            # parity tests reveals that an empty messages list is added
+            result["Messages"] = []
+        return result
+
     @handler("PutMetricAlarm", expand=False)
     def put_metric_alarm(
         self,
@@ -324,7 +360,7 @@ class CloudwatchProvider(CloudwatchApi, ServiceLifecycleHook):
         moto.call_moto(context)
 
         name = request.get("AlarmName")
-        arn = aws_stack.cloudwatch_alarm_arn(name)
+        arn = arns.cloudwatch_alarm_arn(name)
         self.tags.tag_resource(arn, request.get("Tags"))
         self.alarm_scheduler.schedule_metric_alarm(arn)
 
@@ -384,3 +420,16 @@ class CloudwatchProvider(CloudwatchApi, ServiceLifecycleHook):
             _cleanup_describe_output(m)
 
         return response
+
+    @handler("GetMetricStatistics", expand=False)
+    def get_metric_statistics(
+        self, context: RequestContext, request: GetMetricStatisticsInput
+    ) -> GetMetricStatisticsOutput:
+        response = moto.call_moto(context)
+
+        # cleanup -> ExtendendStatics is not included in AWS response if it returned empty
+        for datapoint in response.get("Datapoints"):
+            if "ExtendedStatistics" in datapoint and not datapoint.get("ExtendedStatistics"):
+                datapoint.pop("ExtendedStatistics")
+
+        return response
