@@ -5,8 +5,11 @@ from typing import Dict, List, Optional
 from cachetools import TTLCache
 from moto.core.exceptions import JsonRESTError
 
+from localstack.aws.accounts import get_aws_account_id
 from localstack.aws.api.dynamodb import ResourceNotFoundException
+from localstack.constants import TEST_AWS_SECRET_ACCESS_KEY
 from localstack.utils.aws import aws_stack
+from localstack.utils.aws.arns import dynamodb_table_arn
 from localstack.utils.json import canonical_json
 from localstack.utils.testutil import list_all_resources
 
@@ -40,8 +43,10 @@ class ItemSet:
 
 class SchemaExtractor:
     @classmethod
-    def extract_keys(cls, item: Dict, table_name: str) -> Optional[Dict]:
-        key_schema = cls.get_key_schema(table_name)
+    def extract_keys(
+        cls, item: Dict, table_name: str, account_id: str = None, region_name: str = None
+    ) -> Optional[Dict]:
+        key_schema = cls.get_key_schema(table_name, account_id, region_name)
         return cls.extract_keys_for_schema(item, key_schema)
 
     @classmethod
@@ -58,10 +63,18 @@ class SchemaExtractor:
         return result
 
     @classmethod
-    def get_key_schema(cls, table_name: str) -> Optional[List[Dict]]:
+    def get_key_schema(
+        cls, table_name: str, account_id: str = None, region_name: str = None
+    ) -> Optional[List[Dict]]:
         from localstack.services.dynamodb.provider import get_store
 
-        table_definitions: Dict = get_store().table_definitions
+        account_id = account_id or get_aws_account_id()
+        region_name = region_name or aws_stack.get_region()
+
+        table_definitions: Dict = get_store(
+            account_id=account_id,
+            region_name=region_name,
+        ).table_definitions
         table_def = table_definitions.get(table_name)
         if not table_def:
             raise ResourceNotFoundException(
@@ -70,12 +83,21 @@ class SchemaExtractor:
         return table_def["KeySchema"]
 
     @classmethod
-    def get_table_schema(cls, table_name):
-        key = f"{aws_stack.get_region()}/{table_name}"
+    def get_table_schema(cls, table_name: str, account_id: str = None, region_name: str = None):
+        account_id = account_id or get_aws_account_id()
+        region_name = region_name or aws_stack.get_region()
+        key = dynamodb_table_arn(
+            table_name=table_name, account_id=account_id, region_name=region_name
+        )
         schema = SCHEMA_CACHE.get(key)
         if not schema:
             # TODO: consider making in-memory lookup instead of API call
-            ddb_client = aws_stack.connect_to_service("dynamodb")
+            ddb_client = aws_stack.connect_to_service(
+                "dynamodb",
+                aws_access_key_id=account_id,
+                aws_secret_access_key=TEST_AWS_SECRET_ACCESS_KEY,
+                region_name=region_name,
+            )
             try:
                 schema = ddb_client.describe_table(TableName=table_name)
                 SCHEMA_CACHE[key] = schema
@@ -92,7 +114,12 @@ class ItemFinder:
         from localstack.services.dynamodb.provider import ValidationException
 
         table_name = table_name or put_item["TableName"]
-        ddb_client = aws_stack.connect_to_service("dynamodb")
+        ddb_client = aws_stack.connect_to_service(
+            "dynamodb",
+            aws_access_key_id=get_aws_account_id(),
+            aws_secret_access_key=TEST_AWS_SECRET_ACCESS_KEY,
+            region_name=aws_stack.get_region(),
+        )
 
         search_key = {}
         if "Key" in put_item:
@@ -117,7 +144,7 @@ class ItemFinder:
                 return
 
         req = {"TableName": table_name, "Key": search_key}
-        existing_item = aws_stack.dynamodb_get_item_raw(req)
+        existing_item = ddb_client.get_item(**req)
         if not existing_item:
             return existing_item
         if "Item" not in existing_item:
@@ -144,7 +171,12 @@ class ItemFinder:
 
     @staticmethod
     def get_all_table_items(table_name: str) -> List:
-        ddb_client = aws_stack.connect_to_service("dynamodb")
+        ddb_client = aws_stack.connect_to_service(
+            "dynamodb",
+            aws_access_key_id=get_aws_account_id(),
+            aws_secret_access_key=TEST_AWS_SECRET_ACCESS_KEY,
+            region_name=aws_stack.get_region(),
+        )
         dynamodb_kwargs = {"TableName": table_name}
         all_items = list_all_resources(
             lambda kwargs: ddb_client.scan(**{**kwargs, **dynamodb_kwargs}),
