@@ -1,23 +1,21 @@
 import json
-import re
 from datetime import datetime
 from typing import Dict, List
 from urllib.parse import quote
 
-from moto.iam.models import AWSManagedPolicy, InlinePolicy, Policy
+from moto.iam.models import AWSManagedPolicy, IAMBackend, InlinePolicy, Policy
 from moto.iam.models import Role as MotoRole
-from moto.iam.models import aws_managed_policies, filter_items_with_path_prefix
-from moto.iam.models import iam_backends as moto_iam_backends
-from moto.iam.policy_validation import VALID_STATEMENT_ELEMENTS, IAMPolicyDocumentValidator
-from moto.iam.responses import IamResponse
+from moto.iam.models import filter_items_with_path_prefix, iam_backends
+from moto.iam.policy_validation import VALID_STATEMENT_ELEMENTS
 
-from localstack.aws.accounts import get_aws_account_id
-from localstack.aws.api import CommonServiceException, RequestContext
+from localstack.aws.api import CommonServiceException, RequestContext, handler
 from localstack.aws.api.iam import (
     ActionNameListType,
     ActionNameType,
     AttachedPermissionsBoundary,
     ContextEntryListType,
+    CreateRoleRequest,
+    CreateRoleResponse,
     CreateServiceLinkedRoleResponse,
     CreateUserResponse,
     DeleteServiceLinkedRoleResponse,
@@ -90,10 +88,33 @@ ADDITIONAL_MANAGED_POLICIES = {
 }
 
 
+def get_iam_backend(context: RequestContext) -> IAMBackend:
+    return iam_backends[context.account_id]["global"]
+
+
 class IamProvider(IamApi):
     def __init__(self):
         apply_patches()
 
+    @handler("CreateRole", expand=False)
+    def create_role(
+        self, context: RequestContext, request: CreateRoleRequest
+    ) -> CreateRoleResponse:
+        result = call_moto(context)
+
+        if not request.get("MaxSessionDuration") and result["Role"].get("MaxSessionDuration"):
+            backend = get_iam_backend(context)
+            role = backend.get_role(request["RoleName"])
+            role.max_session_duration = None
+            result["Role"].pop("MaxSessionDuration")
+
+        if "RoleLastUsed" in result["Role"] and not result["Role"]["RoleLastUsed"]:
+            # not part of the AWS response if it's empty
+            # FIXME: RoleLastUsed did not seem well supported when this check was added
+            result["Role"].pop("RoleLastUsed")
+
+        return result
+
     @staticmethod
     def build_evaluation_result(
         action_name: ActionNameType, resource_name: ResourceNameType, policy_statements: List[Dict]
@@ -129,10 +150,9 @@ class IamProvider(IamApi):
         max_items: maxItemsType = None,
         marker: markerType = None,
     ) -> SimulatePolicyResponse:
-        policy = iam_global_backend().get_policy(policy_source_arn)
-        policy_version = iam_global_backend().get_policy_version(
-            policy_source_arn, policy.default_version_id
-        )
+        backend = get_iam_backend(context)
+        policy = backend.get_policy(policy_source_arn)
+        policy_version = backend.get_policy_version(policy_source_arn, policy.default_version_id)
         try:
             policy_statements = json.loads(policy_version.document).get("Statement", [])
         except Exception:
@@ -150,16 +170,18 @@ class IamProvider(IamApi):
         return response
 
     def delete_policy(self, context: RequestContext, policy_arn: arnType) -> None:
-        if iam_global_backend().managed_policies.get(policy_arn):
-            iam_global_backend().managed_policies.pop(policy_arn, None)
+        backend = get_iam_backend(context)
+        if backend.managed_policies.get(policy_arn):
+            backend.managed_policies.pop(policy_arn, None)
         else:
             raise NoSuchEntityException("Policy {0} was not found.".format(policy_arn))
 
     def detach_role_policy(
         self, context: RequestContext, role_name: roleNameType, policy_arn: arnType
     ) -> None:
+        backend = get_iam_backend(context)
         try:
-            role = iam_global_backend().get_role(role_name)
+            role = backend.get_role(role_name)
             policy = role.managed_policies[policy_arn]
             policy.detach_from(role)
         except KeyError:
@@ -193,7 +215,8 @@ class IamProvider(IamApi):
         marker: markerType = None,
         max_items: maxItemsType = None,
     ) -> ListRolesResponse:
-        moto_roles = iam_global_backend().roles.values()
+        backend = get_iam_backend(context)
+        moto_roles = backend.roles.values()
         if path_prefix:
             moto_roles = filter_items_with_path_prefix(path_prefix, moto_roles)
         moto_roles = sorted(moto_roles, key=lambda role: role.id)
@@ -219,10 +242,11 @@ class IamProvider(IamApi):
         new_group_name: groupNameType = None,
     ) -> None:
         new_group_name = new_group_name or group_name
-        group = iam_global_backend().get_group(group_name)
+        backend = get_iam_backend(context)
+        group = backend.get_group(group_name)
         group.path = new_path
         group.name = new_group_name
-        iam_global_backend().groups[new_group_name] = iam_global_backend().groups.pop(group_name)
+        backend.groups[new_group_name] = backend.groups.pop(group_name)
 
     def list_instance_profile_tags(
         self,
@@ -231,7 +255,8 @@ class IamProvider(IamApi):
         marker: markerType = None,
         max_items: maxItemsType = None,
     ) -> ListInstanceProfileTagsResponse:
-        profile = iam_global_backend().get_instance_profile(instance_profile_name)
+        backend = get_iam_backend(context)
+        profile = backend.get_instance_profile(instance_profile_name)
         response = ListInstanceProfileTagsResponse()
         response["Tags"] = [Tag(Key=k, Value=v) for k, v in profile.tags.items()]
         return response
@@ -242,7 +267,8 @@ class IamProvider(IamApi):
         instance_profile_name: instanceProfileNameType,
         tags: tagListType,
     ) -> None:
-        profile = iam_global_backend().get_instance_profile(instance_profile_name)
+        backend = get_iam_backend(context)
+        profile = backend.get_instance_profile(instance_profile_name)
         value_by_key = {tag["Key"]: tag["Value"] for tag in tags}
         profile.tags.update(value_by_key)
 
@@ -252,7 +278,8 @@ class IamProvider(IamApi):
         instance_profile_name: instanceProfileNameType,
         tag_keys: tagKeyListType,
     ) -> None:
-        profile = iam_global_backend().get_instance_profile(instance_profile_name)
+        backend = get_iam_backend(context)
+        profile = backend.get_instance_profile(instance_profile_name)
         for tag in tag_keys:
             profile.tags.pop(tag, None)
 
@@ -279,7 +306,8 @@ class IamProvider(IamApi):
         )
         path = f"{SERVICE_LINKED_ROLE_PATH_PREFIX}/{aws_service_name}"
         role_name = f"r-{short_uid()}"
-        role = iam_global_backend().create_role(
+        backend = get_iam_backend(context)
+        role = backend.create_role(
             role_name=role_name,
             assume_role_policy_document=policy_doc,
             path=path,
@@ -289,7 +317,7 @@ class IamProvider(IamApi):
             max_session_duration=3600,
         )
         role.service_linked_role_arn = "arn:aws:iam::{0}:role/aws-service-role/{1}/{2}".format(
-            get_aws_account_id(), aws_service_name, role.name
+            context.account_id, aws_service_name, role.name
         )
 
         res_role = self.moto_role_to_role_type(role)
@@ -299,7 +327,8 @@ class IamProvider(IamApi):
         self, context: RequestContext, role_name: roleNameType
     ) -> DeleteServiceLinkedRoleResponse:
         # TODO: test
-        iam_global_backend().delete_role(role_name)
+        backend = get_iam_backend(context)
+        backend.delete_role(role_name)
         return DeleteServiceLinkedRoleResponse(DeletionTaskId=short_uid())
 
     def get_service_linked_role_deletion_status(
@@ -311,7 +340,7 @@ class IamProvider(IamApi):
     def put_user_permissions_boundary(
         self, context: RequestContext, user_name: userNameType, permissions_boundary: arnType
     ) -> None:
-        if user := iam_global_backend().users.get(user_name):
+        if user := get_iam_backend(context).users.get(user_name):
             user.permissions_boundary = permissions_boundary
         else:
             raise NoSuchEntityException()
@@ -319,7 +348,7 @@ class IamProvider(IamApi):
     def delete_user_permissions_boundary(
         self, context: RequestContext, user_name: userNameType
     ) -> None:
-        if user := iam_global_backend().users.get(user_name):
+        if user := get_iam_backend(context).users.get(user_name):
             if hasattr(user, "permissions_boundary"):
                 delattr(user, "permissions_boundary")
         else:
@@ -334,7 +363,7 @@ class IamProvider(IamApi):
         tags: tagListType = None,
     ) -> CreateUserResponse:
         response = call_moto(context=context)
-        user = iam_global_backend().get_user(user_name)
+        user = get_iam_backend(context).get_user(user_name)
         if permissions_boundary:
             user.permissions_boundary = permissions_boundary
             response["User"]["PermissionsBoundary"] = AttachedPermissionsBoundary(
@@ -348,7 +377,7 @@ class IamProvider(IamApi):
     ) -> GetUserResponse:
         response = call_moto(context=context)
         moto_user_name = response["User"]["UserName"]
-        moto_user = iam_global_backend().users.get(moto_user_name)
+        moto_user = get_iam_backend(context).users.get(moto_user_name)
         # if the user does not exist or is no user
         if not moto_user and not user_name:
             access_key_id = extract_access_key_id_from_auth_header(context.request.headers)
@@ -418,10 +447,6 @@ class IamProvider(IamApi):
     #     return GetUserResponse(User=response_user)
 
 
-def iam_global_backend():
-    return moto_iam_backends["global"]
-
-
 def apply_patches():
     # support service linked roles
 
@@ -433,54 +458,33 @@ def apply_patches():
     MotoRole.arn = moto_role_arn
 
     # Add missing managed polices
-
-    aws_managed_policies.extend(
-        [AWSManagedPolicy.from_data(k, v) for k, v in ADDITIONAL_MANAGED_POLICIES.items()]
-    )
+    @patch(IAMBackend.__init__)
+    def add_attributes(__init__, self, region_name, account_id, aws_policies=None):
+        __init__(self, region_name, account_id, aws_policies=aws_policies)
+        self.aws_managed_policies.extend(
+            [
+                AWSManagedPolicy.from_data(name, self.account_id, d)
+                for name, d in ADDITIONAL_MANAGED_POLICIES.items()
+            ]
+        )
 
     if "Principal" not in VALID_STATEMENT_ELEMENTS:
         VALID_STATEMENT_ELEMENTS.append("Principal")
 
-    @patch(IAMPolicyDocumentValidator._validate_resource_syntax, pass_target=False)
-    def _validate_resource_syntax(statement, *args, **kwargs):
-        # Note: Serverless generates policies without "Resource" section (only "Effect"/"Principal"/"Action"),
-        # which causes several policy validators in moto to fail
-        if statement.get("Resource") in [None, [None]]:
-            statement["Resource"] = ["*"]
-
-    # patch get_user to include tags
-    # TODO: remove this patch in favour of IamProvider.get_user.
-
-    @patch(IamResponse.get_user)
-    def iam_response_get_user(fn, self):
-        result = fn(self)
-        regex = r"(.*<UserName>\s*)([^\s]+)(\s*</UserName>.*)"
-        flags = re.MULTILINE | re.DOTALL
-
-        user_name = re.match(regex, result, flags=flags).group(2)
-        # replace default user id/name in response
-
-        user = iam_global_backend().users.get(user_name)
-        if not user:
-            return result
-        tags = iam_global_backend().tagger.list_tags_for_resource(user.arn)
-        if tags and "<Tags>" not in result:
-            tags_str = "".join(
-                [
-                    "<member><Key>%s</Key><Value>%s</Value></member>" % (t["Key"], t["Value"])
-                    for t in tags["Tags"]
-                ]
-            )
-            result = result.replace("</Arn>", "</Arn><Tags>%s</Tags>" % tags_str)
-        return result
-
     # patch policy __init__ to set document as attribute
 
     @patch(Policy.__init__)
     def policy__init__(
-        fn, self, name, default_version_id=None, description=None, document=None, **kwargs
+        fn,
+        self,
+        name,
+        account_id,
+        default_version_id=None,
+        description=None,
+        document=None,
+        **kwargs,
     ):
-        fn(self, name, default_version_id, description, document, **kwargs)
+        fn(self, name, account_id, default_version_id, description, document, **kwargs)
         self.document = document
 
     # patch unapply_policy
