@@ -17,15 +17,13 @@ from localstack import config, constants
 from localstack.aws.accounts import get_aws_account_id
 from localstack.constants import ENV_DEV, LOCALSTACK_INFRA_PROCESS, LOCALSTACK_VENV_FOLDER
 from localstack.runtime import events, hooks
-from localstack.services import generic_proxy, install, motoserver
+from localstack.services import generic_proxy, motoserver
 from localstack.services.generic_proxy import ProxyListener, start_proxy_server
 from localstack.services.plugins import SERVICE_PLUGINS, ServiceDisabled, wait_for_infra_shutdown
 from localstack.utils import analytics, config_listener, files, persistence
-from localstack.utils.analytics import event_publisher
 from localstack.utils.aws.request_context import patch_moto_request_handling
 from localstack.utils.bootstrap import (
     canonicalize_api_names,
-    in_ci,
     is_api_enabled,
     log_duration,
     setup_logging,
@@ -164,6 +162,7 @@ def do_run(
             env_vars=env_vars,
             auto_restart=auto_restart,
             strip_color=strip_color,
+            name="todo_dorun",
         )
         t.start()
         TMP_THREADS.append(t)
@@ -273,7 +272,7 @@ def start_local_api(name, port, api, method, asynchronous=False, listener=None):
         port = get_free_tcp_port()
         PROXY_LISTENERS[api] = (api, port, listener)
     if asynchronous:
-        thread = start_thread(method, port, quiet=True)
+        thread = start_thread(method, port, quiet=True, name=f"aws-api-{api}")
         return thread
     else:
         method(port)
@@ -282,11 +281,14 @@ def start_local_api(name, port, api, method, asynchronous=False, listener=None):
 def stop_infra():
     if events.infra_stopping.is_set():
         return
+
+    analytics.log.event("infra_stop")
+
     # also used to signal shutdown for edge proxy so that any further requests will be rejected
     events.infra_stopping.set()
 
-    event_publisher.fire_event(event_publisher.EVENT_STOP_INFRA)
-    analytics.log.event("infra_stop")
+    # run plugin hooks for infra shutdown
+    hooks.on_infra_shutdown.run()
 
     try:
         generic_proxy.QUIET = True  # TODO: this doesn't seem to be doing anything
@@ -437,13 +439,10 @@ def start_infra(asynchronous=False, apis=None):
 
 
 def do_start_infra(asynchronous, apis, is_in_docker):
-    event_publisher.fire_event(
-        event_publisher.EVENT_START_INFRA,
-        {"d": is_in_docker and 1 or 0, "c": in_ci() and 1 or 0},
-    )
-
     if config.DEVELOP:
-        install.install_debugpy_and_dependencies()
+        from localstack.packages.debugpy import debugpy_package
+
+        debugpy_package.install()
         import debugpy
 
         LOG.info("Starting debug server at: %s:%s", constants.BIND_HOST, config.DEVELOP_PORT)
@@ -468,11 +467,6 @@ def do_start_infra(asynchronous, apis, is_in_docker):
         check_aws_credentials()
         patch_moto_request_handling()
 
-    @log_duration()
-    def prepare_installation():
-        # install libs if not present
-        install.install_components(apis)
-
     @log_duration()
     def preload_services():
         """
@@ -517,7 +511,6 @@ def do_start_infra(asynchronous, apis, is_in_docker):
         return t
 
     prepare_environment()
-    prepare_installation()
     thread = start_runtime_components()
     preload_services()
 
