@@ -3,11 +3,11 @@ This module provides tools to call moto using moto and botocore internals withou
 """
 import sys
 from functools import lru_cache
-from typing import Callable
+from typing import Callable, Optional, Union
 
-from moto.backends import get_backend as get_moto_backend
+import moto.backends as moto_backends
+from moto.core import BackendDict
 from moto.core.exceptions import RESTError
-from moto.core.utils import BackendDict
 from moto.moto_server.utilities import RegexConverter
 from werkzeug.exceptions import NotFound
 from werkzeug.routing import Map, Rule
@@ -21,9 +21,13 @@ from localstack.aws.api import (
     ServiceRequest,
     ServiceResponse,
 )
-from localstack.aws.client import parse_response, raise_service_exception
-from localstack.aws.forwarder import ForwardingFallbackDispatcher, create_aws_request_context
+from localstack.aws.forwarder import (
+    ForwardingFallbackDispatcher,
+    create_aws_request_context,
+    dispatch_to_backend,
+)
 from localstack.aws.skeleton import DispatchTable
+from localstack.constants import DEFAULT_AWS_ACCOUNT_ID
 from localstack.http import Response
 
 MotoDispatcher = Callable[[HttpRequest, str, dict], Response]
@@ -37,15 +41,9 @@ def call_moto(context: RequestContext, include_response_metadata=False) -> Servi
 
     :param context: the request context
     :param include_response_metadata: whether to include botocore's "ResponseMetadata" attribute
-    :return: an AWS ServiceResponse (same as a service provider would return)
-    :raises ServiceException: if moto returned an error response
+    :return: a serialized AWS ServiceResponse (same as boto3 would return)
     """
-    status, headers, content = dispatch_to_moto(context)
-    response = Response(content, status, headers)
-    parsed_response = parse_response(context.operation, response, include_response_metadata)
-    raise_service_exception(response, parsed_response)
-
-    return parsed_response
+    return dispatch_to_backend(context, dispatch_to_moto, include_response_metadata)
 
 
 def call_moto_with_request(
@@ -72,18 +70,17 @@ def call_moto_with_request(
     return call_moto(local_context)
 
 
-def proxy_moto(context: RequestContext, service_request: ServiceRequest = None) -> Response:
+def _proxy_moto(
+    context: RequestContext, request: ServiceRequest
+) -> Optional[Union[ServiceResponse, Response]]:
     """
-    Similar to ``call``, only that ``proxy`` does not parse the HTTP response into a ServiceResponse, but instead
-    returns directly the HTTP response. This can be useful to pass through moto's response directly to the client.
+    Wraps `call_moto` such that the interface is compliant with a ServiceRequestHandler.
 
     :param context: the request context
     :param service_request: currently not being used, added to satisfy ServiceRequestHandler contract
     :return: the Response from moto
     """
-    status, headers, content = dispatch_to_moto(context)
-
-    return Response(response=content, status=status, headers=headers)
+    return call_moto(context)
 
 
 def MotoFallbackDispatcher(provider: object) -> DispatchTable:
@@ -95,7 +92,7 @@ def MotoFallbackDispatcher(provider: object) -> DispatchTable:
     :param provider: the ASF provider
     :return: a modified DispatchTable
     """
-    return ForwardingFallbackDispatcher(provider, proxy_moto)
+    return ForwardingFallbackDispatcher(provider, _proxy_moto)
 
 
 def dispatch_to_moto(context: RequestContext) -> Response:
@@ -111,7 +108,12 @@ def dispatch_to_moto(context: RequestContext) -> Response:
     dispatch = get_dispatcher(service.service_name, request.path)
 
     try:
-        return dispatch(request, request.url, request.headers)
+        status, headers, content = dispatch(request, request.url, request.headers)
+        if isinstance(content, str) and len(content) == 0:
+            # moto often returns an empty string to indicate an empty body.
+            # use None instead to ensure that body-related headers aren't overwritten when creating the response object.
+            content = None
+        return Response(content, status, headers)
     except RESTError as e:
         raise CommonServiceException(e.error_type, e.message, status_code=e.code) from e
 
@@ -149,9 +151,14 @@ def load_moto_routing_table(service: str) -> Map:
     :return: a new Map object
     """
     # code from moto.moto_server.werkzeug_app.create_backend_app
-    backend_dict: BackendDict = get_moto_backend(service)
-    if "us-east-1" in backend_dict:
-        backend = backend_dict["us-east-1"]
+    backend_dict = moto_backends.get_backend(service)
+    # Get an instance of this backend.
+    # We'll only use this backend to resolve the URL's, so the exact region/account_id is irrelevant
+    if isinstance(backend_dict, BackendDict):
+        if "us-east-1" in backend_dict[DEFAULT_AWS_ACCOUNT_ID]:
+            backend = backend_dict[DEFAULT_AWS_ACCOUNT_ID]["us-east-1"]
+        else:
+            backend = backend_dict[DEFAULT_AWS_ACCOUNT_ID]["global"]
     else:
         backend = backend_dict["global"]
 
