@@ -129,6 +129,10 @@ class ServiceLifecycleHook:
     def on_before_stop(self):
         pass
 
+    def on_after_inject(self):
+        """Hook triggered after new state has been injected into the provider's store."""
+        pass
+
     def on_exception(self):
         pass
 
@@ -522,6 +526,20 @@ class ServicePluginManager(ServiceManager):
             if self.get_active_provider(service) in providers
         ]
 
+    def _get_loaded_service_containers(
+        self, services: Optional[List[str]] = None
+    ) -> List[ServiceContainer]:
+        """
+        Returns all the available service containers.
+        :param services: the list of services to restrict the search to. If empty or NULL then service containers for
+                         all available services are queried.
+        :return: a list of all the available service containers.
+        """
+        services = services or self.list_available()
+        return [
+            c for s in services if (c := super(ServicePluginManager, self).get_service_container(s))
+        ]
+
     def list_loaded_services(self) -> List[str]:
         """
         Lists all the services which have a provider that has been initialized
@@ -529,9 +547,20 @@ class ServicePluginManager(ServiceManager):
         :return: a list of service names
         """
         return [
-            service
-            for service in self.list_available()
-            if super(ServicePluginManager, self).get_service_container(service)
+            service_container.service.name()
+            for service_container in self._get_loaded_service_containers()
+        ]
+
+    def list_active_services(self) -> List[str]:
+        """
+        Lists all services that have an initialised provider and are currently running.
+
+        :return: the list of active service names.
+        """
+        return [
+            service_container.service.name()
+            for service_container in self._get_loaded_service_containers()
+            if service_container.state == ServiceState.RUNNING
         ]
 
     def exists(self, name: str) -> bool:
@@ -640,6 +669,17 @@ class ServicePluginManager(ServiceManager):
                 apis.append(api)
         return apis
 
+    def _stop_services(self, service_containers: List[ServiceContainer]) -> None:
+        """
+        Atomically attempts to stop all given 'ServiceState.STARTING' and 'ServiceState.RUNNING' services.
+        :param service_containers: the list of service containers to be stopped.
+        """
+        target_service_states = {ServiceState.STARTING, ServiceState.RUNNING}
+        with self._mutex:
+            for service_container in service_containers:
+                if service_container.state in target_service_states:
+                    service_container.stop()
+
     def stop_services(self, services: List[str] = None):
         """
         Stops services for this service manager, if they are currently active.
@@ -647,18 +687,16 @@ class ServicePluginManager(ServiceManager):
 
         :param services: Service names to stop. If not provided, all services for this manager will be stopped.
         """
-        for service_name in services:
-            if self.get_state(service_name) in [ServiceState.STARTING, ServiceState.RUNNING]:
-                service_container = self.get_service_container(service_name)
-                service_container.stop()
+        target_service_containers = self._get_loaded_service_containers(services=services)
+        self._stop_services(target_service_containers)
 
     def stop_all_services(self) -> None:
         """
         Stops all services for this service manager, if they are currently active.
         Will not stop services not already started or in and error state.
         """
-        services = self.list_available()
-        self.stop_services(services)
+        target_service_containers = self._get_loaded_service_containers()
+        self._stop_services(target_service_containers)
 
 
 # map of service plugins, mapping from service name to plugin details
@@ -708,16 +746,21 @@ def local_api_checker(service: str) -> Callable:
         return lambda *args, **kwargs: None
 
     def _check(expect_shutdown=False, print_error=False):
+        port = None
         try:
             if service not in PROXY_LISTENERS:
                 LOG.debug("cannot find backend port for service %s", service)
                 return
             port = PROXY_LISTENERS[service][1]
 
+            if port is None:
+                # for modern ASF services, the port can be none since the service is just served by localstack
+                return
+
             LOG.debug("checking service health %s:%d", service, port)
             wait_for_port_status(port, expect_success=not expect_shutdown)
         except Exception:
             if print_error:
-                LOG.exception("service health check %s:%d failed", service, port)
+                LOG.exception("service health check %s:%s failed", service, port)
 
     return _check
