@@ -4,6 +4,7 @@ from moto.ec2.models import ec2_backends
 from moto.route53resolver.models import Route53ResolverBackend as MotoRoute53ResolverBackend
 from moto.route53resolver.models import route53resolver_backends
 
+import localstack.services.route53resolver.utils
 from localstack.aws.api import RequestContext
 from localstack.aws.api.route53resolver import (
     Action,
@@ -16,6 +17,7 @@ from localstack.aws.api.route53resolver import (
     CreateFirewallDomainListResponse,
     CreateFirewallRuleGroupResponse,
     CreateFirewallRuleResponse,
+    CreateResolverEndpointResponse,
     CreateResolverQueryLogConfigResponse,
     CreatorRequestId,
     DeleteFirewallDomainListResponse,
@@ -43,6 +45,9 @@ from localstack.aws.api.route53resolver import (
     GetFirewallRuleGroupResponse,
     GetResolverQueryLogConfigAssociationResponse,
     GetResolverQueryLogConfigResponse,
+    InvalidParameterException,
+    InvalidRequestException,
+    IpAddressesRequest,
     ListDomainMaxResults,
     ListFirewallConfigsMaxResult,
     ListFirewallConfigsResponse,
@@ -57,12 +62,15 @@ from localstack.aws.api.route53resolver import (
     Name,
     NextToken,
     Priority,
+    ResolverEndpointDirection,
     ResolverQueryLogConfig,
     ResolverQueryLogConfigAssociation,
     ResolverQueryLogConfigName,
+    ResolverQueryLogConfigStatus,
     ResourceId,
     ResourceNotFoundException,
     Route53ResolverApi,
+    SecurityGroupIds,
     SortByKey,
     SortOrder,
     TagList,
@@ -72,7 +80,8 @@ from localstack.aws.api.route53resolver import (
     UpdateFirewallRuleResponse,
     ValidationException,
 )
-from localstack.services.route53resolver.models import Route53ResolverBackend
+from localstack.services.moto import call_moto
+from localstack.services.route53resolver.models import Route53ResolverStore, route53resolver_stores
 from localstack.services.route53resolver.utils import (
     get_resolver_query_log_config_id,
     get_route53_resolver_firewall_domain_list_id,
@@ -83,12 +92,17 @@ from localstack.services.route53resolver.utils import (
     validate_mutation_protection,
     validate_priority,
 )
-from localstack.utils.aws import aws_stack
+from localstack.utils.aws import arns
+from localstack.utils.aws.arns import extract_account_id_from_arn, extract_region_from_arn
 from localstack.utils.collections import select_from_typed_dict
 from localstack.utils.patch import patch
 
 
 class Route53ResolverProvider(Route53ResolverApi):
+    @staticmethod
+    def get_store(account_id: str, region: str) -> Route53ResolverStore:
+        return route53resolver_stores[account_id][region]
+
     def create_firewall_rule_group(
         self,
         context: RequestContext,
@@ -97,9 +111,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         tags: TagList = None,
     ) -> CreateFirewallRuleGroupResponse:
         """Create a Firewall Rule Group."""
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         id = get_route53_resolver_firewall_rule_group_id()
-        arn = aws_stack.get_route53_resolver_firewall_rule_group_arn(id)
+        arn = arns.get_route53_resolver_firewall_rule_group_arn(id)
         firewall_rule_group = FirewallRuleGroup(
             Id=id,
             Arn=arn,
@@ -113,7 +127,7 @@ class Route53ResolverProvider(Route53ResolverApi):
             CreationTime=datetime.now(timezone.utc).isoformat(),
             ModificationTime=datetime.now(timezone.utc).isoformat(),
         )
-        region_details.firewall_rule_groups[id] = firewall_rule_group
+        store.firewall_rule_groups[id] = firewall_rule_group
         route53resolver_backends[context.account_id][context.region].tagger.tag_resource(
             arn, tags or []
         )
@@ -122,9 +136,9 @@ class Route53ResolverProvider(Route53ResolverApi):
     def delete_firewall_rule_group(
         self, context: RequestContext, firewall_rule_group_id: ResourceId
     ) -> DeleteFirewallRuleGroupResponse:
-        region_details = Route53ResolverBackend.get()
         """Delete a Firewall Rule Group."""
-        firewall_rule_group: FirewallRuleGroup = region_details.delete_firewall_rule_group(
+        store = self.get_store(context.account_id, context.region)
+        firewall_rule_group: FirewallRuleGroup = store.delete_firewall_rule_group(
             firewall_rule_group_id
         )
         return DeleteFirewallRuleGroupResponse(FirewallRuleGroup=firewall_rule_group)
@@ -132,9 +146,9 @@ class Route53ResolverProvider(Route53ResolverApi):
     def get_firewall_rule_group(
         self, context: RequestContext, firewall_rule_group_id: ResourceId
     ) -> GetFirewallRuleGroupResponse:
-        region_details = Route53ResolverBackend.get()
         """Get the details of a Firewall Rule Group."""
-        firewall_rule_group: FirewallRuleGroup = region_details.get_firewall_rule_group(
+        store = self.get_store(context.account_id, context.region)
+        firewall_rule_group: FirewallRuleGroup = store.get_firewall_rule_group(
             firewall_rule_group_id
         )
         return GetFirewallRuleGroupResponse(FirewallRuleGroup=firewall_rule_group)
@@ -143,9 +157,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         self, context: RequestContext, max_results: MaxResults = None, next_token: NextToken = None
     ) -> ListFirewallRuleGroupsResponse:
         """List Firewall Rule Groups."""
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         firewall_rule_groups = []
-        for firewall_rule_group in region_details.firewall_rule_groups.values():
+        for firewall_rule_group in store.firewall_rule_groups.values():
             firewall_rule_groups.append(
                 select_from_typed_dict(FirewallRuleGroupMetadata, firewall_rule_group)
             )
@@ -159,9 +173,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         tags: TagList = None,
     ) -> CreateFirewallDomainListResponse:
         """Create a Firewall Domain List."""
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         id = get_route53_resolver_firewall_domain_list_id()
-        arn = aws_stack.get_route53_resolver_firewall_domain_list_arn(id)
+        arn = arns.get_route53_resolver_firewall_domain_list_arn(id)
         firewall_domain_list = FirewallDomainList(
             Id=id,
             Arn=arn,
@@ -174,7 +188,7 @@ class Route53ResolverProvider(Route53ResolverApi):
             CreationTime=datetime.now(timezone.utc).isoformat(),
             ModificationTime=datetime.now(timezone.utc).isoformat(),
         )
-        region_details.firewall_domain_lists[id] = firewall_domain_list
+        store.firewall_domain_lists[id] = firewall_domain_list
         route53resolver_backends[context.account_id][context.region].tagger.tag_resource(
             arn, tags or []
         )
@@ -183,9 +197,9 @@ class Route53ResolverProvider(Route53ResolverApi):
     def delete_firewall_domain_list(
         self, context: RequestContext, firewall_domain_list_id: ResourceId
     ) -> DeleteFirewallDomainListResponse:
-        region_details = Route53ResolverBackend.get()
         """Delete a Firewall Domain List."""
-        firewall_domain_list: FirewallDomainList = region_details.delete_firewall_domain_list(
+        store = self.get_store(context.account_id, context.region)
+        firewall_domain_list: FirewallDomainList = store.delete_firewall_domain_list(
             firewall_domain_list_id
         )
         return DeleteFirewallDomainListResponse(FirewallDomainList=firewall_domain_list)
@@ -193,9 +207,9 @@ class Route53ResolverProvider(Route53ResolverApi):
     def get_firewall_domain_list(
         self, context: RequestContext, firewall_domain_list_id: ResourceId
     ) -> GetFirewallDomainListResponse:
-        region_details = Route53ResolverBackend.get()
         """Get the details of a Firewall Domain List."""
-        firewall_domain_list: FirewallDomainList = region_details.get_firewall_domain_list(
+        store = self.get_store(context.account_id, context.region)
+        firewall_domain_list: FirewallDomainList = store.get_firewall_domain_list(
             firewall_domain_list_id
         )
         return GetFirewallDomainListResponse(FirewallDomainList=firewall_domain_list)
@@ -204,9 +218,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         self, context: RequestContext, max_results: MaxResults = None, next_token: NextToken = None
     ) -> ListFirewallDomainListsResponse:
         """List all Firewall Domain Lists."""
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         firewall_domain_lists = []
-        for firewall_domain_list in region_details.firewall_domain_lists.values():
+        for firewall_domain_list in store.firewall_domain_lists.values():
             firewall_domain_list.append(
                 select_from_typed_dict(FirewallDomainListMetadata, firewall_domain_list)
             )
@@ -219,23 +233,19 @@ class Route53ResolverProvider(Route53ResolverApi):
         operation: FirewallDomainUpdateOperation,
         domains: FirewallDomains,
     ) -> UpdateFirewallDomainsResponse:
-        region_details = Route53ResolverBackend.get()
         """Update the domains in a Firewall Domain List."""
-        region_details = Route53ResolverBackend.get()
-        firewall_domain_list: FirewallDomainList = region_details.get_firewall_domain_list(
-            firewall_domain_list_id
-        )
+        store = self.get_store(context.account_id, context.region)
 
-        firewall_domain_list: FirewallDomainList = region_details.get_firewall_domain_list(
+        firewall_domain_list: FirewallDomainList = store.get_firewall_domain_list(
             firewall_domain_list_id
         )
-        firewall_domains = region_details.get_firewall_domain(firewall_domain_list_id)
+        firewall_domains = store.get_firewall_domain(firewall_domain_list_id)
 
         if operation == FirewallDomainUpdateOperation.ADD:
             if not firewall_domains:
-                region_details.firewall_domains[firewall_domain_list_id] = domains
+                store.firewall_domains[firewall_domain_list_id] = domains
             else:
-                region_details.firewall_domains[firewall_domain_list_id].append(domains)
+                store.firewall_domains[firewall_domain_list_id].append(domains)
 
         if operation == FirewallDomainUpdateOperation.REMOVE:
             if firewall_domains:
@@ -244,11 +254,11 @@ class Route53ResolverProvider(Route53ResolverApi):
                         firewall_domains.remove(domain)
                     else:
                         raise ValidationException(
-                            f"[RSLVR-02502] The following domains don't exist in the DNS Firewall domain list '{firewall_domain_list_id}'. You can't delete a domain that isn't in a domain list. Example unknown domain: '{domain}'. Trace Id: '{aws_stack.get_trace_id()}'"
+                            f"[RSLVR-02502] The following domains don't exist in the DNS Firewall domain list '{firewall_domain_list_id}'. You can't delete a domain that isn't in a domain list. Example unknown domain: '{domain}'. Trace Id: '{localstack.services.route53resolver.utils.get_trace_id()}'"
                         )
 
         if operation == FirewallDomainUpdateOperation.REPLACE:
-            region_details.firewall_domains[firewall_domain_list_id] = domains
+            store.firewall_domains[firewall_domain_list_id] = domains
 
         firewall_domain_list["StatusMessage"] = "Finished domain list update"
         firewall_domain_list["ModificationTime"] = datetime.now(timezone.utc).isoformat()
@@ -267,10 +277,10 @@ class Route53ResolverProvider(Route53ResolverApi):
         next_token: NextToken = None,
     ) -> ListFirewallDomainsResponse:
         """List the domains in a DNS Firewall domain list."""
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         firewall_domains: FirewallDomains[FirewallDomainName] = []
-        if region_details.firewall_domains.get(firewall_domain_list_id):
-            for firewall_domain in region_details.firewall_domains.get(firewall_domain_list_id):
+        if store.firewall_domains.get(firewall_domain_list_id):
+            for firewall_domain in store.firewall_domains.get(firewall_domain_list_id):
                 firewall_domains.append(FirewallDomainName(firewall_domain))
         return ListFirewallDomainsResponse(Domains=firewall_domains)
 
@@ -289,7 +299,7 @@ class Route53ResolverProvider(Route53ResolverApi):
         block_override_ttl: BlockOverrideTtl = None,
     ) -> CreateFirewallRuleResponse:
         """Create a new firewall rule"""
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         firewall_rule = FirewallRule(
             FirewallRuleGroupId=firewall_rule_group_id,
             FirewallDomainListId=firewall_domain_list_id,
@@ -304,15 +314,11 @@ class Route53ResolverProvider(Route53ResolverApi):
             CreationTime=datetime.now(timezone.utc).isoformat(),
             ModificationTime=datetime.now(timezone.utc).isoformat(),
         )
-        if region_details.firewall_rules.get(firewall_rule_group_id):
-            region_details.firewall_rules[firewall_rule_group_id][
-                firewall_domain_list_id
-            ] = firewall_rule
+        if store.firewall_rules.get(firewall_rule_group_id):
+            store.firewall_rules[firewall_rule_group_id][firewall_domain_list_id] = firewall_rule
         else:
-            region_details.firewall_rules[firewall_rule_group_id] = {}
-            region_details.firewall_rules[firewall_rule_group_id][
-                firewall_domain_list_id
-            ] = firewall_rule
+            store.firewall_rules[firewall_rule_group_id] = {}
+            store.firewall_rules[firewall_rule_group_id][firewall_domain_list_id] = firewall_rule
         return CreateFirewallRuleResponse(FirewallRule=firewall_rule)
 
     def delete_firewall_rule(
@@ -321,9 +327,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         firewall_rule_group_id: ResourceId,
         firewall_domain_list_id: ResourceId,
     ) -> DeleteFirewallRuleResponse:
-        region_details = Route53ResolverBackend.get()
         """Delete a firewall rule"""
-        firewall_rule: FirewallRule = region_details.delete_firewall_rule(
+        store = self.get_store(context.account_id, context.region)
+        firewall_rule: FirewallRule = store.delete_firewall_rule(
             firewall_rule_group_id, firewall_domain_list_id
         )
         return DeleteFirewallRuleResponse(
@@ -341,13 +347,13 @@ class Route53ResolverProvider(Route53ResolverApi):
     ) -> ListFirewallRulesResponse:
         """List all the firewall rules in a firewall rule group."""
         # TODO: implement priority and action filtering
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         firewall_rules = []
-        for firewall_rule in region_details.firewall_rules.get(firewall_rule_group_id, {}).values():
+        for firewall_rule in store.firewall_rules.get(firewall_rule_group_id, {}).values():
             firewall_rules.append(FirewallRule(firewall_rule))
         if len(firewall_rules) == 0:
             raise ResourceNotFoundException(
-                f"Can't find the resource with ID '{firewall_rule_group_id}'. Trace Id: '{aws_stack.get_trace_id()}'"
+                f"Can't find the resource with ID '{firewall_rule_group_id}'. Trace Id: '{localstack.services.route53resolver.utils.get_trace_id()}'"
             )
         return ListFirewallRulesResponse(
             FirewallRules=firewall_rules,
@@ -366,9 +372,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         block_override_ttl: BlockOverrideTtl = None,
         name: Name = None,
     ) -> UpdateFirewallRuleResponse:
-        region_details = Route53ResolverBackend.get()
         """Updates a firewall rule"""
-        firewall_rule: FirewallRule = region_details.get_firewall_rule(
+        store = self.get_store(context.account_id, context.region)
+        firewall_rule: FirewallRule = store.get_firewall_rule(
             firewall_rule_group_id, firewall_domain_list_id
         )
 
@@ -402,24 +408,22 @@ class Route53ResolverProvider(Route53ResolverApi):
         tags: TagList = None,
     ) -> AssociateFirewallRuleGroupResponse:
         """Associate a firewall rule group with a VPC."""
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         validate_priority(priority=priority)
         validate_mutation_protection(mutation_protection=mutation_protection)
 
-        for (
-            firewall_rule_group_association
-        ) in region_details.firewall_rule_group_associations.values():
+        for firewall_rule_group_association in store.firewall_rule_group_associations.values():
             if (
                 firewall_rule_group_association.get("VpcId") == vpc_id
                 and firewall_rule_group_association.get("FirewallRuleGroupId")
                 == firewall_rule_group_id
             ):
                 raise ValidationException(
-                    f"[RSLVR-02302] This DNS Firewall rule group can't be associated to a VPC: '{vpc_id}'. It is already associated to VPC '{firewall_rule_group_id}'. Try again with another VPC or DNS Firewall rule group. Trace Id: '{aws_stack.get_trace_id()}'"
+                    f"[RSLVR-02302] This DNS Firewall rule group can't be associated to a VPC: '{vpc_id}'. It is already associated to VPC '{firewall_rule_group_id}'. Try again with another VPC or DNS Firewall rule group. Trace Id: '{localstack.services.route53resolver.utils.get_trace_id()}'"
                 )
 
         id = get_route53_resolver_firewall_rule_group_association_id()
-        arn = aws_stack.get_route53_resolver_firewall_rule_group_associations_arn(id)
+        arn = arns.get_route53_resolver_firewall_rule_group_associations_arn(id)
 
         firewall_rule_group_association = FirewallRuleGroupAssociation(
             Id=id,
@@ -435,7 +439,7 @@ class Route53ResolverProvider(Route53ResolverApi):
             CreationTime=datetime.now(timezone.utc).isoformat(),
             ModificationTime=datetime.now(timezone.utc).isoformat(),
         )
-        region_details.firewall_rule_group_associations[id] = firewall_rule_group_association
+        store.firewall_rule_group_associations[id] = firewall_rule_group_association
         route53resolver_backends[context.account_id][context.region].tagger.tag_resource(
             arn, tags or []
         )
@@ -446,12 +450,10 @@ class Route53ResolverProvider(Route53ResolverApi):
     def disassociate_firewall_rule_group(
         self, context: RequestContext, firewall_rule_group_association_id: ResourceId
     ) -> DisassociateFirewallRuleGroupResponse:
-        region_details = Route53ResolverBackend.get()
         """Disassociate a DNS Firewall rule group from a VPC."""
+        store = self.get_store(context.account_id, context.region)
         firewall_rule_group_association: FirewallRuleGroupAssociation = (
-            region_details.delete_firewall_rule_group_association(
-                firewall_rule_group_association_id
-            )
+            store.delete_firewall_rule_group_association(firewall_rule_group_association_id)
         )
         return DisassociateFirewallRuleGroupResponse(
             FirewallRuleGroupAssociation=firewall_rule_group_association
@@ -460,10 +462,10 @@ class Route53ResolverProvider(Route53ResolverApi):
     def get_firewall_rule_group_association(
         self, context: RequestContext, firewall_rule_group_association_id: ResourceId
     ) -> GetFirewallRuleGroupAssociationResponse:
-        region_details = Route53ResolverBackend.get()
         """Returns the Firewall Rule Group Association that you specified."""
+        store = self.get_store(context.account_id, context.region)
         firewall_rule_group_association: FirewallRuleGroupAssociation = (
-            region_details.get_firewall_rule_group_association(firewall_rule_group_association_id)
+            store.get_firewall_rule_group_association(firewall_rule_group_association_id)
         )
         return GetFirewallRuleGroupAssociationResponse(
             FirewallRuleGroupAssociation=firewall_rule_group_association
@@ -477,13 +479,13 @@ class Route53ResolverProvider(Route53ResolverApi):
         mutation_protection: MutationProtectionStatus = None,
         name: Name = None,
     ) -> UpdateFirewallRuleGroupAssociationResponse:
-        region_details = Route53ResolverBackend.get()
         """Updates the specified Firewall Rule Group Association."""
+        store = self.get_store(context.account_id, context.region)
         validate_priority(priority=priority)
         validate_mutation_protection(mutation_protection=mutation_protection)
 
         firewall_rule_group_association: FirewallRuleGroupAssociation = (
-            region_details.get_firewall_rule_group_association(firewall_rule_group_association_id)
+            store.get_firewall_rule_group_association(firewall_rule_group_association_id)
         )
 
         if priority:
@@ -505,10 +507,10 @@ class Route53ResolverProvider(Route53ResolverApi):
         creator_request_id: CreatorRequestId,
         tags: TagList = None,
     ) -> CreateResolverQueryLogConfigResponse:
+        store = self.get_store(context.account_id, context.region)
         validate_destination_arn(destination_arn)
-        region_details = Route53ResolverBackend.get()
         id = get_resolver_query_log_config_id()
-        arn = aws_stack.get_resolver_query_log_config_arn(id)
+        arn = arns.get_resolver_query_log_config_arn(id)
         resolver_query_log_config: ResolverQueryLogConfig = ResolverQueryLogConfig(
             Id=id,
             Arn=arn,
@@ -521,7 +523,7 @@ class Route53ResolverProvider(Route53ResolverApi):
             CreatorRequestId=creator_request_id,
             CreationTime=datetime.now(timezone.utc).isoformat(),
         )
-        region_details.resolver_query_log_configs[id] = resolver_query_log_config
+        store.resolver_query_log_configs[id] = resolver_query_log_config
         route53resolver_backends[context.account_id][context.region].tagger.tag_resource(
             arn, tags or []
         )
@@ -529,21 +531,37 @@ class Route53ResolverProvider(Route53ResolverApi):
             ResolverQueryLogConfig=resolver_query_log_config
         )
 
+    def create_resolver_endpoint(
+        self,
+        context: RequestContext,
+        creator_request_id: CreatorRequestId,
+        security_group_ids: SecurityGroupIds,
+        direction: ResolverEndpointDirection,
+        ip_addresses: IpAddressesRequest,
+        name: Name = None,
+        tags: TagList = None,
+    ) -> CreateResolverEndpointResponse:
+        create_resolver_endpoint_resp = call_moto(context)
+        create_resolver_endpoint_resp["ResolverEndpoint"][
+            "Status"
+        ] = ResolverQueryLogConfigStatus.CREATING
+        return create_resolver_endpoint_resp
+
     def get_resolver_query_log_config(
         self, context: RequestContext, resolver_query_log_config_id: ResourceId
     ) -> GetResolverQueryLogConfigResponse:
-        region_details = Route53ResolverBackend.get()
-        resolver_query_log_config: ResolverQueryLogConfig = (
-            region_details.get_resolver_query_log_config(resolver_query_log_config_id)
+        store = self.get_store(context.account_id, context.region)
+        resolver_query_log_config: ResolverQueryLogConfig = store.get_resolver_query_log_config(
+            resolver_query_log_config_id
         )
         return GetResolverQueryLogConfigResponse(ResolverQueryLogConfig=resolver_query_log_config)
 
     def delete_resolver_query_log_config(
         self, context: RequestContext, resolver_query_log_config_id: ResourceId
     ) -> DeleteResolverQueryLogConfigResponse:
-        region_details = Route53ResolverBackend.get()
-        resolver_query_log_config: ResolverQueryLogConfig = (
-            region_details.delete_resolver_query_log_config(resolver_query_log_config_id)
+        store = self.get_store(context.account_id, context.region)
+        resolver_query_log_config: ResolverQueryLogConfig = store.delete_resolver_query_log_config(
+            resolver_query_log_config_id
         )
         return DeleteResolverQueryLogConfigResponse(
             ResolverQueryLogConfig=resolver_query_log_config
@@ -558,9 +576,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         sort_by: SortByKey = None,
         sort_order: SortOrder = None,
     ) -> ListResolverQueryLogConfigsResponse:
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         resolver_query_log_configs = []
-        for resolver_query_log_config in region_details.resolver_query_log_configs.values():
+        for resolver_query_log_config in store.resolver_query_log_configs.values():
             resolver_query_log_configs.append(ResolverQueryLogConfig(resolver_query_log_config))
         return ListResolverQueryLogConfigsResponse(
             ResolverQueryLogConfigs=resolver_query_log_configs,
@@ -573,8 +591,7 @@ class Route53ResolverProvider(Route53ResolverApi):
         resolver_query_log_config_id: ResourceId,
         resource_id: ResourceId,
     ) -> AssociateResolverQueryLogConfigResponse:
-
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         id = get_route53_resolver_query_log_config_association_id()
 
         resolver_query_log_config_association: ResolverQueryLogConfigAssociation = (
@@ -589,9 +606,7 @@ class Route53ResolverProvider(Route53ResolverApi):
             )
         )
 
-        region_details.resolver_query_log_config_associations[
-            id
-        ] = resolver_query_log_config_association
+        store.resolver_query_log_config_associations[id] = resolver_query_log_config_association
 
         return AssociateResolverQueryLogConfigResponse(
             ResolverQueryLogConfigAssociation=resolver_query_log_config_association
@@ -603,11 +618,9 @@ class Route53ResolverProvider(Route53ResolverApi):
         resolver_query_log_config_id: ResourceId,
         resource_id: ResourceId,
     ) -> DisassociateResolverQueryLogConfigResponse:
-        region_details = Route53ResolverBackend.get()
-        resolver_query_log_config_association = (
-            region_details.delete_resolver_query_log_config_associations(
-                resolver_query_log_config_id, resource_id
-            )
+        store = self.get_store(context.account_id, context.region)
+        resolver_query_log_config_association = store.delete_resolver_query_log_config_associations(
+            resolver_query_log_config_id, resource_id
         )
 
         return DisassociateResolverQueryLogConfigResponse(
@@ -617,9 +630,9 @@ class Route53ResolverProvider(Route53ResolverApi):
     def get_resolver_query_log_config_association(
         self, context: RequestContext, resolver_query_log_config_association_id: ResourceId
     ) -> GetResolverQueryLogConfigAssociationResponse:
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         resolver_query_log_config_association: ResolverQueryLogConfigAssociation = (
-            region_details.get_resolver_query_log_config_associations(
+            store.get_resolver_query_log_config_associations(
                 resolver_query_log_config_association_id
             )
         )
@@ -636,12 +649,11 @@ class Route53ResolverProvider(Route53ResolverApi):
         sort_by: SortByKey = None,
         sort_order: SortOrder = None,
     ) -> ListResolverQueryLogConfigAssociationsResponse:
-        region_details = Route53ResolverBackend.get()
-
+        store = self.get_store(context.account_id, context.region)
         resolver_query_log_config_associations = []
         for (
             resolver_query_log_config_association
-        ) in region_details.resolver_query_log_config_associations.values():
+        ) in store.resolver_query_log_config_associations.values():
             resolver_query_log_config_associations.append(
                 ResolverQueryLogConfigAssociation(resolver_query_log_config_association)
             )
@@ -653,8 +665,8 @@ class Route53ResolverProvider(Route53ResolverApi):
     def get_firewall_config(
         self, context: RequestContext, resource_id: ResourceId
     ) -> GetFirewallConfigResponse:
-        region_details = Route53ResolverBackend.get()
-        firewall_config = region_details.get_or_create_firewall_config(
+        store = self.get_store(context.account_id, context.region)
+        firewall_config = store.get_or_create_firewall_config(
             resource_id, context.region, context.account_id
         )
         return GetFirewallConfigResponse(FirewallConfig=firewall_config)
@@ -665,15 +677,13 @@ class Route53ResolverProvider(Route53ResolverApi):
         max_results: ListFirewallConfigsMaxResult = None,
         next_token: NextToken = None,
     ) -> ListFirewallConfigsResponse:
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         firewall_configs = []
         backend = ec2_backends[context.account_id][context.region]
         for vpc in backend.vpcs:
-            if vpc not in region_details.firewall_configs:
-                region_details.get_or_create_firewall_config(
-                    vpc, context.region, context.account_id
-                )
-        for firewall_config in region_details.firewall_configs.values():
+            if vpc not in store.firewall_configs:
+                store.get_or_create_firewall_config(vpc, context.region, context.account_id)
+        for firewall_config in store.firewall_configs.values():
             firewall_configs.append(select_from_typed_dict(FirewallConfig, firewall_config))
         return ListFirewallConfigsResponse(FirewallConfigs=firewall_configs)
 
@@ -683,16 +693,16 @@ class Route53ResolverProvider(Route53ResolverApi):
         resource_id: ResourceId,
         firewall_fail_open: FirewallFailOpenStatus,
     ) -> UpdateFirewallConfigResponse:
-        region_details = Route53ResolverBackend.get()
+        store = self.get_store(context.account_id, context.region)
         backend = ec2_backends[context.account_id][context.region]
         for resource_id in backend.vpcs:
-            if resource_id not in region_details.firewall_configs:
-                firewall_config = region_details.get_or_create_firewall_config(
+            if resource_id not in store.firewall_configs:
+                firewall_config = store.get_or_create_firewall_config(
                     resource_id, context.region, context.account_id
                 )
                 firewall_config["FirewallFailOpen"] = firewall_fail_open
             else:
-                firewall_config = region_details.firewall_configs[resource_id]
+                firewall_config = store.firewall_configs[resource_id]
                 firewall_config["FirewallFailOpen"] = firewall_fail_open
         return UpdateFirewallConfigResponse(FirewallConfig=firewall_config)
 
@@ -700,17 +710,63 @@ class Route53ResolverProvider(Route53ResolverApi):
 @patch(MotoRoute53ResolverBackend._matched_arn)
 def Route53ResolverBackend_matched_arn(fn, self, resource_arn):
     """Given ARN, raise exception if there is no corresponding resource."""
-    region_details = Route53ResolverBackend.get()
-    for firewall_rule_group in region_details.firewall_rule_groups.values():
+    account_id = extract_account_id_from_arn(resource_arn)
+    region_name = extract_region_from_arn(resource_arn)
+    store = Route53ResolverProvider.get_store(account_id, region_name)
+
+    for firewall_rule_group in store.firewall_rule_groups.values():
         if firewall_rule_group.get("Arn") == resource_arn:
             return
-    for firewall_domain_list in region_details.firewall_domain_lists.values():
+    for firewall_domain_list in store.firewall_domain_lists.values():
         if firewall_domain_list.get("Arn") == resource_arn:
             return
-    for firewall_rule_group_association in region_details.firewall_rule_group_associations.values():
+    for firewall_rule_group_association in store.firewall_rule_group_associations.values():
         if firewall_rule_group_association.get("Arn") == resource_arn:
             return
-    for resolver_query_log_config in region_details.resolver_query_log_configs.values():
+    for resolver_query_log_config in store.resolver_query_log_configs.values():
         if resolver_query_log_config.get("Arn") == resource_arn:
             return
     fn(self, resource_arn)
+
+
+@patch(MotoRoute53ResolverBackend.disassociate_resolver_rule)
+def moto_disassociate_resolver_rule(fn, self, resolver_rule_id, vpc_id):
+    if resolver_rule_id not in self.resolver_rules:
+        raise ResourceNotFoundException(
+            f'[RSLVR-00703] Resolver rule with ID "{resolver_rule_id}" does not exist.'
+        )
+    return fn(self, resolver_rule_id, vpc_id)
+
+
+@patch(MotoRoute53ResolverBackend.create_resolver_endpoint)
+def moto_create_resolver_endpoint(fn, self, *args, **kwargs):
+    for group_id in kwargs.get("security_group_ids"):
+        if not group_id.startswith("sg-"):
+            raise InvalidParameterException(
+                f'[RSLVR-00408] Malformed security group ID: "Invalid id: "{group_id}" '
+                f'(expecting "sg-...")".'
+            )
+    return fn(self, *args, **kwargs)
+
+
+@patch(MotoRoute53ResolverBackend.delete_resolver_rule)
+def moto_delete_resolver_endpoint(fn, self, resolver_rule_id):
+    if resolver_rule_id not in self.resolver_rules:
+        raise ResourceNotFoundException(
+            f'[RSLVR-00703] Resolver rule with ID "{resolver_rule_id}" does not exist.'
+        )
+    return fn(self, resolver_rule_id)
+
+
+@patch(MotoRoute53ResolverBackend.create_resolver_rule)
+def moto_create_resolver_rule(fn, self, *args, **kwargs):
+    direction = [
+        x.direction
+        for x in self.resolver_endpoints.values()
+        if x.id == kwargs.get("resolver_endpoint_id")
+    ]
+    if direction and direction[0] == ResolverEndpointDirection.INBOUND:
+        raise InvalidRequestException(
+            "[RSLVR-00700] Resolver rules can only be associated to OUTBOUND resolver endpoints."
+        )
+    return fn(self, *args, **kwargs)
