@@ -23,6 +23,7 @@ from localstack import config
 from localstack.aws.accounts import get_aws_account_id
 from localstack.aws.api import RequestContext
 from localstack.aws.api.core import CommonServiceException
+from localstack.aws.api.lambda_ import InvocationType
 from localstack.aws.api.sns import (
     ActionsList,
     AmazonResourceName,
@@ -101,9 +102,9 @@ from localstack.services.edge import ROUTER
 from localstack.services.moto import call_moto
 from localstack.services.plugins import ServiceLifecycleHook
 from localstack.services.sns.models import SnsStore, sns_stores
-from localstack.utils.aws import aws_stack
+from localstack.utils.aws import arns, aws_stack
+from localstack.utils.aws.arns import extract_region_from_arn
 from localstack.utils.aws.aws_responses import create_sqs_system_attributes
-from localstack.utils.aws.aws_stack import extract_region_from_arn
 from localstack.utils.aws.dead_letter_queue import sns_error_to_dead_letter_queue
 from localstack.utils.cloudwatch.cloudwatch_util import store_cloudwatch_logs
 from localstack.utils.json import json_safe
@@ -132,7 +133,8 @@ LOG = logging.getLogger(__name__)
 
 GCM_URL = "https://fcm.googleapis.com/fcm/send"
 
-MSG_ATTR_NAME_REGEX = r"^(?!\.)(?!.*\.$)(?!.*\.\.)[a-zA-Z0-9_\-.]+$"
+MSG_ATTR_NAME_REGEX = re.compile(r"^(?!\.)(?!.*\.$)(?!.*\.\.)[a-zA-Z0-9_\-.]+$")
+ATTR_TYPE_REGEX = re.compile(r"^(String|Number|Binary)\..+$")
 VALID_MSG_ATTR_NAME_CHARS = set(ascii_letters + digits + "." + "-" + "_")
 
 
@@ -163,7 +165,8 @@ def publish_message(
                 message_structure,
                 endpoint_attributes,
                 platform_app,
-            )
+            ),
+            name="sns-message_to_endpoint",
         )
         return message_id
 
@@ -179,7 +182,8 @@ def publish_message(
             subscription_arn,
             skip_checks,
             message_attributes,
-        )
+        ),
+        name="sns-message_to_subscribers",
     )
 
     return message_id
@@ -977,7 +981,7 @@ async def message_to_subscriber(
             elif "://" in endpoint:
                 queue_url = endpoint
             else:
-                queue_url = aws_stack.get_sqs_queue_url(endpoint)
+                queue_url = arns.get_sqs_queue_url(endpoint)
                 subscriber["sqs_queue_url"] = queue_url
 
             message_group_id = req_data.get("MessageGroupId", [""])[0]
@@ -1168,7 +1172,7 @@ async def message_to_subscriber(
             subscriber=subscriber, req_data=req_data, message_id=message_id
         )
         if endpoint:
-            delivery_stream = aws_stack.extract_resource_from_arn(endpoint).split("/")[1]
+            delivery_stream = arns.extract_resource_from_arn(endpoint).split("/")[1]
             firehose_client.put_record(
                 DeliveryStreamName=delivery_stream, Record={"Data": to_bytes(sns_body)}
             )
@@ -1231,7 +1235,9 @@ def process_sns_notification_to_lambda(
     inv_result = lambda_client.invoke(
         FunctionName=func_arn,
         Payload=to_bytes(json.dumps(event)),
-        InvocationType="RequestResponse" if config.SYNCHRONOUS_SNS_EVENTS else "Event",
+        InvocationType=InvocationType.RequestResponse
+        if config.SYNCHRONOUS_SNS_EVENTS
+        else InvocationType.Event,  # DEPRECATED
     )
     status_code = inv_result.get("StatusCode")
     payload = inv_result.get("Payload")
@@ -1374,7 +1380,7 @@ def validate_message_attributes(message_attributes: MessageAttributeMap) -> None
         validate_message_attribute_name(attr_name)
         # `DataType` is a required field for MessageAttributeValue
         data_type = attr["DataType"]
-        if data_type not in ("String", "Number", "Binary", "String.Array"):
+        if data_type not in ("String", "Number", "Binary") and not ATTR_TYPE_REGEX.match(data_type):
             raise InvalidParameterValueException(
                 f"The message attribute '{attr_name}' has an invalid message attribute type, the set of supported type prefixes is Binary, Number, and String."
             )
@@ -1397,7 +1403,7 @@ def validate_message_attribute_name(name: str) -> None:
     :param name: message attribute name
     :raises InvalidParameterValueException: if the name does not conform to the spec
     """
-    if not re.match(MSG_ATTR_NAME_REGEX, name):
+    if not MSG_ATTR_NAME_REGEX.match(name):
         # find the proper exception
         if name[0] == ".":
             raise InvalidParameterValueException(
