@@ -6,6 +6,7 @@ import logging
 import re
 import threading
 import time
+from concurrent.futures.thread import ThreadPoolExecutor
 from queue import Empty
 from typing import Dict, List, Optional
 
@@ -34,6 +35,7 @@ from localstack.aws.api.sqs import (
     GetQueueUrlResult,
     Integer,
     InvalidAttributeName,
+    InvalidBatchEntryId,
     InvalidMessageContents,
     ListDeadLetterSourceQueuesResult,
     ListQueuesResult,
@@ -59,6 +61,7 @@ from localstack.aws.api.sqs import (
     TagKeyList,
     TagMap,
     Token,
+    TooManyEntriesInBatchRequest,
 )
 from localstack.aws.spec import load_service
 from localstack.services.plugins import ServiceLifecycleHook
@@ -75,7 +78,8 @@ from localstack.services.sqs.models import (
 )
 from localstack.services.sqs.utils import generate_message_id
 from localstack.utils.aws import aws_stack
-from localstack.utils.aws.aws_stack import parse_arn
+from localstack.utils.aws.arns import parse_arn
+from localstack.utils.cloudwatch.cloudwatch_util import publish_sqs_metric
 from localstack.utils.run import FuncThread
 from localstack.utils.scheduler import Scheduler
 from localstack.utils.strings import md5
@@ -102,16 +106,39 @@ def assert_queue_name(queue_name: str, fifo: bool = False):
         )
 
 
-def check_message_size(message_body: str, max_message_size: int):
+def check_message_size(
+    message_body: str, message_attributes: MessageBodyAttributeMap, max_message_size: int
+):
     # https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/quotas-messages.html
     error = "One or more parameters are invalid. "
     error += f"Reason: Message must be shorter than {max_message_size} bytes."
 
-    # must encode as utf8 to get correct bytes with len
-    if len(message_body.encode("utf8")) > max_message_size:
+    if (
+        _message_body_size(message_body) + _message_attributes_size(message_attributes)
+        > max_message_size
+    ):
         raise InvalidParameterValue(error)
 
 
+def _message_body_size(body: str):
+    return _bytesize(body)
+
+
+def _message_attributes_size(attributes: MessageBodyAttributeMap):
+    if not attributes:
+        return 0
+    message_attributes_keys_size = sum(_bytesize(k) for k in attributes.keys())
+    message_attributes_values_size = sum(
+        sum(_bytesize(v) for v in attr.values()) for attr in attributes.values()
+    )
+    return message_attributes_keys_size + message_attributes_values_size
+
+
+def _bytesize(value: str | bytes):
+    # must encode as utf8 to get correct bytes with len
+    return len(value.encode("utf8")) if isinstance(value, str) else len(value)
+
+
 def check_message_content(message_body: str):
     error = "Invalid characters found. Valid unicode characters are #x9 | #xA | #xD | #x20 to #xD7FF | #xE000 to #xFFFD | #x10000 to #x10FFFF"
 
@@ -119,6 +146,153 @@ def check_message_content(message_body: str):
         raise InvalidMessageContents(error)
 
 
+class CloudwatchDispatcher:
+    """
+    Dispatches SQS metrics for specific api-calls using a ThreadPool
+    """
+
+    def __init__(self, num_thread: int = 3):
+        self.executor = ThreadPoolExecutor(
+            num_thread, thread_name_prefix="sqs-metrics-cloudwatch-dispatcher"
+        )
+
+    def shutdown(self):
+        self.executor.shutdown(wait=False)
+
+    def dispatch_sqs_metric(
+        self, region: str, queue_name: str, metric: str, value: float = 1, unit: str = "Count"
+    ):
+        """
+        Publishes a metric to Cloudwatch using a Threadpool
+        :param region The region that should be used for Cloudwatch client
+        :param queue_name The name of the queue that the metric belongs to
+        :param metric The name of the metric
+        :param value The value for that metric, default 1
+        :param unit The unit for the value, default "Count"
+        """
+        self.executor.submit(
+            publish_sqs_metric,
+            region=region,
+            queue_name=queue_name,
+            metric=metric,
+            value=value,
+            unit=unit,
+        )
+
+    def dispatch_metric_message_sent(self, queue: SqsQueue, message_body_size: int):
+        """
+        Sends metric 'NumberOfMessagesSent' and 'SentMessageSize' to Cloudwatch
+        :param queue The Queue for which the metric will be send
+        :param message_body_size the size of the message in bytes
+        """
+        self.dispatch_sqs_metric(
+            region=queue.region, queue_name=queue.name, metric="NumberOfMessagesSent"
+        )
+        self.dispatch_sqs_metric(
+            region=queue.region,
+            queue_name=queue.name,
+            metric="SentMessageSize",
+            value=message_body_size,
+            unit="Bytes",
+        )
+
+    def dispatch_metric_message_deleted(self, queue: SqsQueue, deleted: int = 1):
+        """
+        Sends metric 'NumberOfMessagesDeleted' to Cloudwatch
+        :param queue The Queue for which the metric will be sent
+        :param deleted The number of messages that were successfully deleted, default: 1
+        """
+        self.dispatch_sqs_metric(
+            region=queue.region,
+            queue_name=queue.name,
+            metric="NumberOfMessagesDeleted",
+            value=deleted,
+        )
+
+    def dispatch_metric_received(self, queue: SqsQueue, received: int):
+        """
+        Sends metric 'NumberOfMessagesReceived' (if received > 0), or 'NumberOfEmptyReceives' to Cloudwatch
+        :param queue The Queue for which the metric will be send
+        :param received The number of messages that have been received
+        """
+        if received > 0:
+            self.dispatch_sqs_metric(
+                region=queue.region,
+                queue_name=queue.name,
+                metric="NumberOfMessagesReceived",
+                value=received,
+            )
+        else:
+            self.dispatch_sqs_metric(
+                region=queue.region, queue_name=queue.name, metric="NumberOfEmptyReceives"
+            )
+
+
+class CloudwatchPublishWorker:
+    """
+    Regularly publish metrics data about approximate messages to Cloudwatch.
+    Includes: ApproximateNumberOfMessagesVisible, ApproximateNumberOfMessagesNotVisible
+        and ApproximateNumberOfMessagesDelayed
+    """
+
+    def __init__(self) -> None:
+        super().__init__()
+        self.scheduler = Scheduler()
+        self.thread: Optional[FuncThread] = None
+
+    def publish_approximate_cloudwatch_metrics(self):
+        for account_id, region_bundle in sqs_stores.items():
+            for region, store in region_bundle.items():
+                for queue in store.queues.values():
+                    self.publish_approximate_metrics_for_queue_to_cloudwatch(queue)
+
+    def publish_approximate_metrics_for_queue_to_cloudwatch(self, queue):
+        """Publishes the metrics for ApproximateNumberOfMessagesVisible, ApproximateNumberOfMessagesNotVisible
+        and ApproximateNumberOfMessagesDelayed to CloudWatch"""
+        # TODO ApproximateAgeOfOldestMessage is missing
+        #  https://docs.aws.amazon.com/AWSSimpleQueueService/latest/SQSDeveloperGuide/sqs-available-cloudwatch-metrics.html
+        publish_sqs_metric(
+            region=queue.region,
+            queue_name=queue.name,
+            metric="ApproximateNumberOfMessagesVisible",
+            value=queue.approx_number_of_messages,
+        )
+        publish_sqs_metric(
+            region=queue.region,
+            queue_name=queue.name,
+            metric="ApproximateNumberOfMessagesNotVisible",
+            value=queue.approx_number_of_messages_not_visible,
+        )
+        publish_sqs_metric(
+            region=queue.region,
+            queue_name=queue.name,
+            metric="ApproximateNumberOfMessagesDelayed",
+            value=queue.approx_number_of_messages_delayed,
+        )
+
+    def start(self):
+        if self.thread:
+            return
+
+        self.scheduler = Scheduler()
+        self.scheduler.schedule(self.publish_approximate_cloudwatch_metrics, period=60)
+
+        def _run(*_args):
+            self.scheduler.run()
+
+        self.thread = start_thread(_run, name="sqs-approx-metrics-cloudwatch-publisher")
+
+    def stop(self):
+        if self.scheduler:
+            self.scheduler.close()
+
+        if self.thread:
+            self.thread.stop()
+
+        self.thread = None
+        self.scheduler = None
+
+
 class QueueUpdateWorker:
     """
     Regularly re-queues inflight and delayed messages whose visibility timeout has expired or delay deadline has been
@@ -156,7 +330,7 @@ class QueueUpdateWorker:
             def _run(*_args):
                 self.scheduler.run()
 
-            self.thread = start_thread(_run)
+            self.thread = start_thread(_run, name="sqs-queue-update-worker")
 
     def stop(self):
         with self.mutex:
@@ -248,6 +422,8 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
         super().__init__()
         self._mutex = threading.RLock()
         self._queue_update_worker = QueueUpdateWorker()
+        self._cloudwatch_publish_worker = CloudwatchPublishWorker()
+        self._cloudwatch_dispatcher = CloudwatchDispatcher()
 
     @staticmethod
     def get_store(account_id: str = None, region: str = None) -> SqsStore:
@@ -255,9 +431,12 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
 
     def on_before_start(self):
         self._queue_update_worker.start()
+        self._cloudwatch_publish_worker.start()
 
     def on_before_stop(self):
         self._queue_update_worker.stop()
+        self._cloudwatch_publish_worker.stop()
+        self._cloudwatch_dispatcher.shutdown()
 
     def _require_queue(self, context: RequestContext, name: str) -> SqsQueue:
         """
@@ -494,7 +673,7 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
 
         # Have to check the message size here, rather than in _put_message
         # to avoid multiple calls for batch messages.
-        check_message_size(message_body, queue.maximum_message_size)
+        check_message_size(message_body, message_attributes, queue.maximum_message_size)
 
         queue_item = self._put_message(
             queue,
@@ -520,6 +699,11 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
     ) -> SendMessageBatchResult:
         queue = self._resolve_queue(context, queue_url=queue_url)
 
+        if entries and (no_entries := len(entries)) > 10:
+            raise TooManyEntriesInBatchRequest(
+                f"Maximum number of entries per request are 10. You have sent {no_entries}."
+            )
+
         self._assert_batch(entries)
         # check the total batch size first and raise BatchRequestTooLong id > DEFAULT_MAXIMUM_MESSAGE_SIZE.
         # This is checked before any messages in the batch are sent.  Raising the exception here should
@@ -597,6 +781,10 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
             MessageAttributes=message_attributes,
         )
 
+        self._cloudwatch_dispatcher.dispatch_metric_message_sent(
+            queue=queue, message_body_size=len(message_body.encode("utf-8"))
+        )
+
         return queue.put(
             message=message,
             message_deduplication_id=message_deduplication_id,
@@ -621,6 +809,10 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
             wait_time_seconds = queue.wait_time_seconds
 
         num = max_number_of_messages or 1
+        # backdoor to get all messages
+        if num == -1:
+            num = queue.visible.qsize()
+
         block = True if wait_time_seconds else False
         # collect messages
         messages = []
@@ -668,6 +860,8 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
             messages.append(msg)
             num -= 1
 
+        self._cloudwatch_dispatcher.dispatch_metric_received(queue, received=len(messages))
+
         # TODO: how does receiving behave if the queue was deleted in the meantime?
         return ReceiveMessageResult(Messages=messages)
 
@@ -716,6 +910,7 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
     ) -> None:
         queue = self._resolve_queue(context, queue_url=queue_url)
         queue.remove(receipt_handle)
+        self._cloudwatch_dispatcher.dispatch_metric_message_deleted(queue)
 
     def delete_message_batch(
         self,
@@ -725,6 +920,7 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
     ) -> DeleteMessageBatchResult:
         queue = self._resolve_queue(context, queue_url=queue_url)
         self._assert_batch(entries)
+        self._assert_valid_message_ids(entries)
 
         successful = []
         failed = []
@@ -743,6 +939,7 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
                             Message=str(e),
                         )
                     )
+        self._cloudwatch_dispatcher.dispatch_metric_message_deleted(queue, deleted=len(successful))
 
         return DeleteMessageBatchResult(
             Successful=successful,
@@ -876,24 +1073,42 @@ class SqsProvider(SqsApi, ServiceLifecycleHook):
                     "WSDL for a list of valid actions. "
                 )
 
-    def _assert_batch(self, batch: List):
+    def _assert_batch(self, batch: List) -> None:
         if not batch:
             raise EmptyBatchRequest
         visited = set()
         for entry in batch:
-            # TODO: InvalidBatchEntryId
-            if entry["Id"] in visited:
+            entry_id = entry["Id"]
+            if not re.search(r"^[\w-]+$", entry_id) or len(entry_id) > 80:
+                raise InvalidBatchEntryId(
+                    "A batch entry id can only contain alphanumeric characters, hyphens and underscores. "
+                    "It can be at most 80 letters long."
+                )
+            if entry_id in visited:
                 raise BatchEntryIdsNotDistinct()
             else:
-                visited.add(entry["Id"])
+                visited.add(entry_id)
 
     def _assert_valid_batch_size(self, batch: List, max_message_size: int):
-        batch_message_size = sum([len(entry.get("MessageBody").encode("utf8")) for entry in batch])
+        batch_message_size = sum(
+            _message_body_size(entry.get("MessageBody"))
+            + _message_attributes_size(entry.get("MessageAttributes"))
+            for entry in batch
+        )
         if batch_message_size > max_message_size:
             error = f"Batch requests cannot be longer than {max_message_size} bytes."
             error += f" You have sent {batch_message_size} bytes."
             raise BatchRequestTooLong(error)
 
+    def _assert_valid_message_ids(self, batch: List):
+        batch_id_regex = r"^[\w-]{1,80}$"
+        for message in batch:
+            if not re.match(batch_id_regex, message.get("Id", "")):
+                raise InvalidBatchEntryId(
+                    "A batch entry id can only contain alphanumeric characters, "
+                    "hyphens and underscores. It can be at most 80 letters long."
+                )
+
 
 # Method from moto's attribute_md5 of moto/sqs/models.py, separated from the Message Object
 def _create_message_attribute_hash(message_attributes) -> Optional[str]:
