@@ -9,6 +9,7 @@ from typing import Dict, List, Optional
 
 from botocore.response import StreamingBody
 from deepdiff import DeepDiff
+from jsonpath_ng import DatumInContext
 from jsonpath_ng.ext import parse
 
 from localstack.testing.snapshots.transformer import (
@@ -27,7 +28,7 @@ class SnapshotMatchResult:
     def __init__(self, a: dict, b: dict, key: str = ""):
         self.a = a
         self.b = b
-        self.result = DeepDiff(a, b, verbose_level=2)
+        self.result = DeepDiff(a, b, verbose_level=2, view="tree")
         self.key = key
 
     def __bool__(self) -> bool:
@@ -111,7 +112,8 @@ class SnapshotSession:
                     }
                     full_state[self.scope_key] = recorded
                     state_to_dump = json.dumps(full_state, indent=2)
-                    fd.write(state_to_dump)
+                    # add line ending to be compatible with pre-commit-hooks (end-of-file-fixer)
+                    fd.write(f"{state_to_dump}\n")
                 except Exception as e:
                     SNAPSHOT_LOGGER.exception(e)
 
@@ -138,12 +140,14 @@ class SnapshotSession:
 
         self.called_keys.add(key)
 
-        self.observed_state[
-            key
-        ] = obj  # TODO: track them separately since the transformation is now done *just* before asserting
+        # order the obj to guarantee reference replacement works as expected
+        self.observed_state[key] = self._order_dict(obj)
+        # TODO: track them separately since the transformation is now done *just* before asserting
 
-        if not self.update and (not self.recorded_state or not self.recorded_state.get(key)):
-            raise Exception("Please run the test first with --snapshot-update")
+        if not self.update and (not self.recorded_state or self.recorded_state.get(key) is None):
+            raise Exception(
+                f"No state for {self.scope_key} recorded. Please (re-)generate the snapshot for this test."
+            )
 
         # TODO: we should return something meaningful here
         return True
@@ -174,17 +178,28 @@ class SnapshotSession:
 
         # TODO: separate these states
         a_all = self.recorded_state
+        if not self.observed_state:
+            # match was never called, so we must assume this isn't a "real" snapshot test
+            # e.g. test_sqs uses the snapshot fixture to configure it via another fixture on module scope
+            #   but might not use them in some individual tests
+            return []
+
         if not a_all and not self.update:
-            SNAPSHOT_LOGGER.warning(
-                "There is no recorded state yet. Snapshot verification skipped."
+            raise Exception(
+                f"No state for {self.scope_key} recorded. Please (re-)generate the snapshot for this test."
             )
-            return results
 
         self._remove_skip_verification_paths(a_all)
         self.observed_state = b_all = self._transform(self.observed_state)
 
         for key in self.called_keys:
-            a = a_all[key]
+            a = a_all.get(
+                key
+            )  # if this is None, a new key was added since last updating => usage error
+            if a is None:
+                raise Exception(
+                    f"State for {key=} missing in {self.scope_key}. Please (re-)generate the snapshot for this test."
+                )
             b = b_all[key]
             result = SnapshotMatchResult(a, b, key=key)
             results.append(result)
@@ -218,14 +233,13 @@ class SnapshotSession:
         """build a persistable state definition that can later be compared against"""
         self._transform_dict_to_parseable_values(tmp)
 
-        if not self.update:
-            self._remove_skip_verification_paths(tmp)
-
         ctx = TransformContext()
-
         for transformer, _ in sorted(self.transformers, key=lambda p: p[1]):
             tmp = transformer.transform(tmp, ctx=ctx)
 
+        if not self.update:
+            self._remove_skip_verification_paths(tmp)
+
         tmp = json.dumps(tmp, default=str)
         for sr in ctx.serialized_replacements:
             tmp = sr(tmp)
@@ -239,8 +253,26 @@ class SnapshotSession:
 
         return tmp
 
-    # LEGACY API
+    def _order_dict(self, response) -> dict:
+        if isinstance(response, dict):
+            ordered_dict = {}
+            for key, val in sorted(response.items()):
+                if isinstance(val, dict):
+                    ordered_dict[key] = self._order_dict(val)
+                elif isinstance(val, list):
+                    ordered_dict[key] = [self._order_dict(entry) for entry in val]
+                else:
+                    ordered_dict[key] = val
 
+            # put the ResponseMetadata back at the end of the response
+            if "ResponseMetadata" in ordered_dict:
+                ordered_dict["ResponseMetadata"] = ordered_dict.pop("ResponseMetadata")
+
+            return ordered_dict
+        else:
+            return response
+
+    # LEGACY API
     def register_replacement(self, pattern: Pattern[str], value: str):
         self.add_transformer(RegexTransformer(pattern, value))
 
@@ -264,10 +296,21 @@ class SnapshotSession:
 
     def _remove_skip_verification_paths(self, tmp: Dict):
         """Removes all keys from the dict, that match the given json-paths in self.skip_verification_path"""
+
+        def build_full_path_nodes(field_match: DatumInContext):
+            """Traverse the matched Datum to build the path field by field"""
+            full_path_nodes = [str(field_match.path)]
+            next_node = field_match
+            while next_node.context is not None:
+                full_path_nodes.append(str(next_node.context.path))
+                next_node = next_node.context
+
+            return full_path_nodes[::-1][1:]  # reverse the list and remove Root()/$
+
         for path in self.skip_verification_paths:
             matches = parse(path).find(tmp) or []
             for m in matches:
-                full_path = str(m.full_path).split(".")
+                full_path = build_full_path_nodes(m)
                 helper = tmp
                 if len(full_path) > 1:
                     for p in full_path[:-1]:
