@@ -27,6 +27,8 @@ THREAD_LOCAL = threading.local()
 
 MARKER_APIGW_REQUEST_REGION = "__apigw_request_region__"
 
+AWS_REGION_REGEX = r"(us(-gov)?|ap|ca|cn|eu|sa)-(central|(north|south)?(east|west)?)-\d"
+
 
 def get_proxy_request_for_thread():
     try:
@@ -181,7 +183,7 @@ def patch_moto_request_handling():
             service = extract_service_name_from_auth_header(request.headers)
             msg = (
                 f"API action '{action}' for service '{service}' not yet implemented or pro feature"
-                f" - check https://docs.localstack.cloud/aws/feature-coverage for further information"
+                f" - check https://docs.localstack.cloud/user-guide/aws/feature-coverage for further information"
             )
             response = requests_error_response(request.headers, msg, code=501)
             if config.MOCK_UNIMPLEMENTED:
