@@ -273,6 +273,7 @@ class PortMappings:
 
 
 SimpleVolumeBind = Tuple[str, str]
+"""Type alias for a simple version of VolumeBind"""
 
 
 @dataclasses.dataclass
@@ -283,7 +284,7 @@ class VolumeBind:
 
     host_dir: str
     container_dir: str
-    options: Optional[List[str]] = None
+    read_only: bool = False
 
     def to_str(self) -> str:
         args = []
@@ -296,8 +297,8 @@ class VolumeBind:
 
         args.append(self.container_dir)
 
-        if self.options:
-            args.append(self.options)
+        if self.read_only:
+            args.append("ro")
 
         return ":".join(args)
 
@@ -673,6 +674,7 @@ class ContainerClient(metaclass=ABCMeta):
             dns=container_config.dns,
             additional_flags=container_config.additional_flags,
             workdir=container_config.workdir,
+            privileged=container_config.privileged,
         )
 
     @abstractmethod
@@ -687,7 +689,7 @@ class ContainerClient(metaclass=ABCMeta):
         tty: bool = False,
         detach: bool = False,
         command: Optional[Union[List[str], str]] = None,
-        mount_volumes: Optional[List[SimpleVolumeBind]] = None,
+        mount_volumes: Optional[Union[VolumeMappings, List[SimpleVolumeBind]]] = None,
         ports: Optional[PortMappings] = None,
         env_vars: Optional[Dict[str, str]] = None,
         user: Optional[str] = None,
@@ -698,6 +700,7 @@ class ContainerClient(metaclass=ABCMeta):
         dns: Optional[str] = None,
         additional_flags: Optional[str] = None,
         workdir: Optional[str] = None,
+        privileged: Optional[bool] = None,
     ) -> str:
         """Creates a container with the given image
 
@@ -729,6 +732,7 @@ class ContainerClient(metaclass=ABCMeta):
         dns: Optional[str] = None,
         additional_flags: Optional[str] = None,
         workdir: Optional[str] = None,
+        privileged: Optional[bool] = None,
     ) -> Tuple[bytes, bytes]:
         """Creates and runs a given docker container
 
@@ -769,6 +773,17 @@ class ContainerClient(metaclass=ABCMeta):
         """
         pass
 
+    @abstractmethod
+    def login(self, username: str, password: str, registry: Optional[str] = None) -> None:
+        """
+        Login into an OCI registry
+
+        :param username: Username for the registry
+        :param password: Password / token for the registry
+        :param registry: Registry url
+        """
+        pass
+
 
 class Util:
     MAX_ENV_ARGS_LENGTH = 20000
@@ -957,12 +972,22 @@ class Util:
 
     @staticmethod
     def convert_mount_list_to_dict(
-        mount_volumes: List[SimpleVolumeBind],
+        mount_volumes: Union[List[SimpleVolumeBind], VolumeMappings],
     ) -> Dict[str, Dict[str, str]]:
         """Converts a List of (host_path, container_path) tuples to a Dict suitable as volume argument for docker sdk"""
+
+        def _map_to_dict(paths: SimpleVolumeBind | VolumeBind):
+            if isinstance(paths, VolumeBind):
+                return str(paths.host_dir), {
+                    "bind": paths.container_dir,
+                    "mode": "ro" if paths.read_only else "rw",
+                }
+            else:
+                return str(paths[0]), {"bind": paths[1], "mode": "rw"}
+
         return dict(
             map(
-                lambda paths: (str(paths[0]), {"bind": paths[1], "mode": "rw"}),
+                _map_to_dict,
                 mount_volumes,
             )
         )
