@@ -22,6 +22,7 @@ from localstack.utils.container_utils.container_client import (
     RegistryConnectionError,
     SimpleVolumeBind,
     Util,
+    VolumeBind,
 )
 from localstack.utils.run import run
 from localstack.utils.strings import to_str
@@ -462,6 +463,19 @@ class CmdDockerClient(ContainerClient):
                     "Docker process returned with errorcode %s" % e.returncode, e.stdout, e.stderr
                 ) from e
 
+    def login(self, username: str, password: str, registry: Optional[str] = None) -> None:
+        cmd = self._docker_cmd()
+        # TODO specify password via stdin
+        cmd += ["login", "-u", username, "-p", password]
+        if registry:
+            cmd.append(registry)
+        try:
+            run(cmd)
+        except subprocess.CalledProcessError as e:
+            raise ContainerException(
+                "Docker process returned with errorcode %s" % e.returncode, e.stdout, e.stderr
+            ) from e
+
     def has_docker(self) -> bool:
         try:
             run(self._docker_cmd() + ["ps"])
@@ -615,8 +629,8 @@ class CmdDockerClient(ContainerClient):
         if mount_volumes:
             cmd += [
                 volume
-                for host_path, docker_path in dict(mount_volumes).items()
-                for volume in ["-v", f"{host_path}:{docker_path}"]
+                for mount_volume in mount_volumes
+                for volume in ["-v", self._map_to_volume_param(mount_volume)]
             ]
         if interactive:
             cmd.append("--interactive")
@@ -651,3 +665,20 @@ class CmdDockerClient(ContainerClient):
         if command:
             cmd += command if isinstance(command, List) else [command]
         return cmd, env_file
+
+    @staticmethod
+    def _map_to_volume_param(mount_volume: Union[SimpleVolumeBind, VolumeBind]) -> str:
+        """
+        Maps the mount volume, to a parameter for the -v docker cli argument.
+
+        Examples:
+        (host_path, container_path) -> host_path:container_path
+        VolumeBind(host_dir=host_path, container_dir=container_path, read_only=True) -> host_path:container_path:ro
+
+        :param mount_volume: Either a SimpleVolumeBind, in essence a tuple (host_dir, container_dir), or a VolumeBind object
+        :return: String which is passable as parameter to the docker cli -v option
+        """
+        if isinstance(mount_volume, VolumeBind):
+            return mount_volume.to_str()
+        else:
+            return f"{mount_volume[0]}:{mount_volume[1]}"
